# -*- coding: utf-8 -*-
#
# Copyright (c) 2009 by FlashCode <flashcode@flashtux.org>
# Copyright (c) 2009 by xt <xt@bash.no>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

#
# Bar with URLs (easy click on long URLs)
# (this script requires WeeChat 0.3.0 or newer)
#
# History:
# 2015-08-25, Simmo Saan <simmo.saan@gmail.com>
#     version 12: fix error on empty prefix
# 2014-03-01, Lars Kiesow <lkiesow@uos.de>
#     version 11: Fixed autocompletion of /urlbar arguments
# 2010-12-20, xt <xt@bash.no>
#     version 10: use API for nick color, strip nick prefix
# 2009-12-17, FlashCode <flashcode@flashtux.org>
#     version 0.9: fix option name "show_index" (spaces removed)
# 2009-12-12, FlashCode <flashcode@flashtux.org>
#     version 0.8: update WeeChat site
# 2009-11-05, xt <xt@bash.no>
#     version 0.7: config option to turn off index
# 2009-10-20, xt <xt@bash.no>
#     version 0.6: removed priority on the bar
# 2009-07-01, xt <xt@bash.no>
#     version 0.5: changed script command to /urlbar, comma separated ignore list
# 2009-05-22, xt <xt@bash.no>
#     version 0.4: added configurable showing of buffer name, nick and time
# 2009-05-21, xt <xt@bash.no>
#     version 0.3: bug fixes, add ignore feature from sleo
# 2009-05-19, xt <xt@bash.no>
#     version 0.2-dev: fixes
# 2009-05-04, FlashCode <flashcode@flashtux.org>
#     version 0.1-dev: dev snapshot
#

SCRIPT_NAME    = "urlbar"
SCRIPT_AUTHOR  = "FlashCode <flashcode@flashtux.org>"
SCRIPT_VERSION = "12"
SCRIPT_LICENSE = "GPL3"
SCRIPT_DESC    = "Bar with URLs. For easy clicking or selecting."
SCRIPT_COMMAND = "urlbar"

settings = {
    "visible_amount"        : '5',     # Amount of URLS visible in urlbar at any given time
    "visible_seconds"       : '5',     # Amount of seconds URLbar is visible
    "use_popup"             : 'on',    # Pop up automatically
    "remember_amount"       : '25',    # Max amout of URLs to keep in RAM
    "ignore"                : 'grep',  # List of buffers to ignore. (comma separated)
    "show_timestamp"        : 'on',    # Show timestamp in list
    "show_nick"             : 'on',    # Show nick in list
    "show_buffername"       : 'on',    # Show buffer name in list
    "show_index"            : 'on',    # Show url index in list
    "time_format"           : '%H:%M', # Time format
}

import_ok = True
try:
    import weechat
except ImportError:
    print "This script must be run under WeeChat."
    print "Get WeeChat now at: http://www.weechat.org/"
    import_ok = False

import re
from time import strftime, localtime
octet = r'(?:2(?:[0-4]\d|5[0-5])|1\d\d|\d{1,2})'
ipAddr = r'%s(?:\.%s){3}' % (octet, octet)
# Base domain regex off RFC 1034 and 1738
label = r'[0-9a-z][-0-9a-z]*[0-9a-z]?'
domain = r'%s(?:\.%s)*\.[a-z][-0-9a-z]*[a-z]?' % (label, label)
urlRe = re.compile(r'(\w+://(?:%s|%s)(?::\d+)?(?:/[^\])>\s]*)?)' % (domain, ipAddr), re.I)


# list of URL-objects
urls = []

# Display ALL, a toggle
DISPLAY_ALL = False



def urlbar_item_cb(data, item, window):
    ''' Callback that prints the lines in the urlbar '''
    global DISPLAY_ALL, urls
    try:
        visible_amount = int(weechat.config_get_plugin('visible_amount'))
    except ValueError:
        weechat.prnt('', 'Invalid value for visible_amount setting.')

    if not urls:
        return 'Empty URL list'

    if DISPLAY_ALL:
        DISPLAY_ALL = False
        printlist = urls
    else:
        printlist = urls[-visible_amount:]

    result = ''
    for index, url in enumerate(printlist):
        if weechat.config_get_plugin('show_index') == 'on':
            index = index+1
            result += '%s%2d%s %s \r' %\
                (weechat.color("yellow"), index, weechat.color("bar_fg"), url)
        else:
            result += '%s%s \r' %(weechat.color('bar_fg'), url)
    return result


def get_buffer_name(bufferp, long=False):
    if not weechat.buffer_get_string(bufferp, "short_name") or long:
        bufferd = weechat.buffer_get_string(bufferp, "name")
    else:
        bufferd = weechat.buffer_get_string(bufferp, "short_name")
    return bufferd

class URL(object):
    ''' URL class that holds the urls in the URL list '''

    def __init__(self, url, buffername, timestamp, nick):
        self.url = url
        self.buffername = buffername
        self.time = strftime(
                weechat.config_get_plugin('time_format'),
                localtime(int(timestamp)))
        self.time = self.time.replace(':', '%s:%s' %
                (weechat.color(weechat.config_string(
                weechat.config_get('weechat.color.chat_time_delimiters'))),
                weechat.color('reset')))
        self.nick = irc_nick_find_color(nick.strip('%&@+'))

    def __str__(self):
        # Format options
        time, buffername, nick = '', '', ''
        if weechat.config_get_plugin('show_timestamp') == 'on':
            time = self.time + ' '
        if weechat.config_get_plugin('show_buffername') == 'on':
            buffername = self.buffername + ' '
        if weechat.config_get_plugin('show_nick') == 'on':
            nick = self.nick + ' '

        return '%s%s%s%s' % (time, nick, buffername, self.url)

    def __cmp__(this, other):
        if this.url == other.url:
            return 0
        return 1

def urlbar_print_cb(data, buffer, time, tags, displayed, highlight, prefix, message):


    buffer_name = get_buffer_name(buffer, long=True)
    # Skip ignored buffers
    for ignored_buffer in weechat.config_get_plugin('ignore').split(','):
        if ignored_buffer.lower() == buffer_name.lower():
            return weechat.WEECHAT_RC_OK

    # Clean list of URLs
    for i in range(len(urls) - int(weechat.config_get_plugin('remember_amount'))):
        # Delete the oldest
        urls.pop(0)

    for url in urlRe.findall(message):
        urlobject = URL(url, get_buffer_name(buffer), time, prefix)
        # Do not add duplicate URLs
        if urlobject in urls:
            continue
        urls.append(urlobject)
        if weechat.config_get_plugin('use_popup') == 'on':
            weechat.command("", "/bar show urlbar")
            # auto hide bar after delay
            try:
                weechat.command('', '/wait %s /bar hide urlbar' %
                        int(weechat.config_get_plugin('visible_seconds')))
            except ValueError:
                weechat.prnt('', 'Invalid visible_seconds')

        weechat.bar_item_update("urlbar_urls")

    return weechat.WEECHAT_RC_OK


def urlbar_cmd(data, buffer, args):
    """ Callback for /url command. """
    global urls, DISPLAY_ALL

    if args == "list":
        if urls:
            DISPLAY_ALL = True
            weechat.command("", '/bar show urlbar')
            weechat.bar_item_update("urlbar_urls")
        else:
            weechat.prnt('', 'URL list empty.')
    if args == "show":
        weechat.command('', '/bar show urlbar')
    elif args == 'hide':
        weechat.command("", "/bar hide urlbar")
    elif args == 'toggle':
        weechat.command("", "/bar toggle urlbar")
    elif args == 'clear':
        urls = []
    elif not args.startswith('url '):
        weechat.command("", "/help %s" % SCRIPT_COMMAND)

    return weechat.WEECHAT_RC_OK

def urlbar_completion_urls_cb(data, completion_item, buffer, completion):
    """ Complete with URLS, for command '/url'. """
    for url in urls:
        weechat.hook_completion_list_add(completion, url.url,
                                         0, weechat.WEECHAT_LIST_POS_SORT)
    return weechat.WEECHAT_RC_OK

def irc_nick_find_color(nick):
    if not nick: # nick (actually prefix) is empty, irc_nick_color returns None on empty input
        return ''

    color = weechat.info_get('irc_nick_color', nick)
    if not color:
        # probably we're in WeeChat 0.3.0
        color = 0
        for char in nick:
            color += ord(char)
        
        color %= weechat.config_integer(weechat.config_get("weechat.look.color_nicks_number"))
        color = weechat.config_get('weechat.color.chat_nick_color%02d' %(color+1))
        color = w.color(weechat.config_string(color))
    return '%s%s%s' %(color, nick, weechat.color('reset'))


if __name__ == "__main__" and import_ok:
    if weechat.register(SCRIPT_NAME, SCRIPT_AUTHOR, SCRIPT_VERSION, SCRIPT_LICENSE,
                        SCRIPT_DESC, "", ""):
        # Set default settings
        for option, default_value in settings.iteritems():
            if not weechat.config_is_set_plugin(option):
                weechat.config_set_plugin(option, default_value)

        weechat.hook_command(SCRIPT_COMMAND,
                             "URL bar control",
                             "[list | hide | show | toggle | url URL]",
                             "   list: list all URL and show URL bar\n"
                             "   hide: hide URL bar\n"
                             "   show: show URL bar\n"
                             "   toggle: toggle showing of URL bar\n",
                             "list || hide || show || toggle || url %(urlbar_urls)",
                             "urlbar_cmd", "")
        weechat.hook_completion("urlbar_urls", "list of URLs",
                                "urlbar_completion_urls_cb", "")
        weechat.bar_item_new("urlbar_urls", "urlbar_item_cb", "");
        weechat.bar_new("urlbar", "on", "0", "root", "", "top", "horizontal",
                        "vertical", "0", "0", "default", "default", "default", "0",
                        "urlbar_urls");
        weechat.hook_print("", "", "://", 1, "urlbar_print_cb", "")



# Copyright (c) 2011 by Stephan Huebner <shuebnerfun01@gmx.org>
#
# Intended use:
#
#     Send an autorespond-message once somebody sends a message in a query, but
#     only send it every n minutes
#
# TODO: make the text appear on highlights only?
#       make the script work for chats too (if own nick is mentioned)?
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# History:
#
#  - Sample History-Entry

SCR_NAME    = "arespond"
SCR_AUTHOR  = "Stephan Huebner <shuebnerfun01@gmx.org>"
SCR_VERSION = "0.1.0"
SCR_LICENSE = "GPL3"
SCR_DESC    = "An autoresponder (sending a notice on other users' messages)"
SCR_COMMAND = "arespond"

import_ok = True

try:
   import weechat as w
except:
   print "Script must be run under weechat. http://www.weechat.org"
   import_ok = False

import time

settings = {
   "responderText" : "Hello. %s isn't available at the moment. This message " +
                     "won't appear anymore for the next %d minutes.",
   "respondAfterMinutes" : "10",
   "muted" :      "off"
}

def errMsg(myMsg):
   alert("ERR: " + myMsg)
   return

def fn_privmsg(data, bufferp, tm, tags, display, is_hilight, prefix, msg):
   global settings
   servername = (w.buffer_get_string(bufferp, "name").split("."))[0]
   ownNick = w.info_get("irc_nick", servername)
   if prefix != ownNick and settings["muted"] != "off":
      # alert("messagetags: " + tags)
      if w.buffer_get_string(bufferp, "localvar_type") == "private":
         oldTime = w.buffer_get_string(bufferp, "localvar_btime")
         rmdTxt = settings["responderText"].replace("%s", ownNick)
         rmdTxt = rmdTxt.replace("%d", settings["respondAfterMinutes"])
         if oldTime != "":
            nowTime = time.time()
            tdelta = int(nowTime)-int(oldTime)
            if int(settings["respondAfterMinutes"])*60 <= tdelta:
               w.command("", "/notice " + prefix + " " + rmdTxt)
               w.buffer_set(bufferp, "localvar_set_btime", str(int(nowTime)))
         else:
            w.buffer_set(bufferp, "localvar_set_btime", str(int(time.time())))
            w.command("", "/notice " + prefix + " " + rmdTxt)
   return w.WEECHAT_RC_OK

def fn_command(data, buffer, args):
   # args being something like "on 15 SomeText"
   args = args.split()
   for listIndex in range(len(args)):
      if "on" in args[listIndex] or "off" in args[listIndex]:
         w.config_set_plugin("muted", args[listIndex])
      else:
         try:
            myMinutes = float(args[listIndex])
            w.config_set_plugin("respondAfterMinutes", args[listIndex])
         except ValueError:
            w.config_set_plugin("responderText", " ".join(args[listIndex:]))
            break
   return w.WEECHAT_RC_OK

def alert(myString):
	w.prnt("", myString)
	return

def fn_configchange(data, option, value):
   global settings
   fields = option.split(".")
   myOption = fields[-1]
   try:
      settings[myOption] = value
      alert("Option {0} is now {1}".format(myOption, settings[myOption]))
   except KeyError:
      errMsg("There is no option named %s" %myOption)
   return w.WEECHAT_RC_OK

if __name__ == "__main__" and import_ok:
   if w.register(SCR_NAME, SCR_AUTHOR, SCR_VERSION, SCR_LICENSE,
                 SCR_DESC, "", ""):
      # synchronize weechat- and scriptsettings
      for option, default_value in settings.items():
         if not w.config_is_set_plugin(option):
            w.config_set_plugin(option, default_value)
         else:
            settings[option] = w.config_get_plugin(option)
      w.hook_print("", "", "", 1, "fn_privmsg", "") # catch prvmsg
      w.hook_config("plugins.var.python." + SCR_NAME + ".*",
                    "fn_configchange", "") # catch configchanges
      w.hook_command(SCR_COMMAND, SCR_DESC, "[muted] [n] [text]",
"""
Available options are:
- muted:               can be "on" or "off"
- respondAfterMinutes: integer (in minutes), after which responderText is
                       sent again
- responderText:       Text to be shown when necessary conditions are met.
                       Attention: The text always has to be the last parameter.
                       It does not need to be in quotes of any kind. It can also
                       contain %s and %d. These are placeholders for your own
                       nick and the minutes set in preferences.
""",
         "", "fn_command", ""
      )



# -*- coding: utf-8 -*-
#
# auto_away.py : A simple auto-away script for Weechat in Python 
# Copyright (c) 2010 by Specimen <spinifer at gmail dot com>
#
# Inspired in yaaa.pl by jnbek
# A very special thanks to Nils G. for helping me out with this script
# ---------------------------------------------------------------------
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
#
# This script requires WeeChat 0.3.0 or newer.
# 
# ---------------------------------------------------------------------
#
# Summary:
#
#   Sets status to away automatically after a given period of 
#   inactivity. Returns from away when you start typing, but not when
#   you change buffer or scroll. This script should work with any
#   plugin and it's not IRC specific.
#
#
# Configuration:
#
#   /autoaway [time|off] [message]
#
#      time: minutes of inactivity to set away
#       off: disable auto-away (0 also disables)
#   message: away message (optional)
#
#   Without any arguments prints the current settings.
#
#
# Configuration options via /set:
#
# 'idletime'
#   description: Period in minutes (n) of keyboard inactivity until 
#                being marked as being away.
#                Setting idletime to "0", a negative number or a string
#                such as "off" disables auto-away. All positive values 
#                are treated as they integers.
#   command: /set plugins.var.python.auto_away.idletime n
# 
# 'message'
#   description: Away message. The /away command requires this setting 
#                not to be empty.
#   command: /set plugins.var.python.auto_away.message "message"
#
#
# Changelog:
#
#   2010-02-11 - 0.1    - Specimen:
#                         Script created.
#   2010-02-11 - 0.1.1  - Specimen
#                         Various fixes with the help of Flashcode
#   2010-02-13 - 0.2    - Specimen:
#                         Option to disable autoaway, return from away
#                         via hook_signal as suggested by Nils G.
#                         No longer uses plugin configuration to store
#                         away status.
#   2010-02-20 - 0.2.5  - Specimen:
#                         Use hook_config to check idletime and
#                         enable/disable hook_timer.
#                         Removed away_status.  
#   2010-02-21 - 0.3    - Specimen:
#                         Implemented /autoaway command.
#   2010-02-22 - 0.3.3  - Specimen:
#                         Fixed /autoaway command args.
#                         Removed hookinterval.
#                         When setting away it now checks if 'message'
#                         is empty and fixes it.
#                         Workaround for /away -all bug in disconnected
#                         servers (uses some screen_away.py code).
#                         /autoaway without arguments outputs current
#                         settings.
#                         Code rewrite.

try:
    import weechat as w
    
except Exception:
    print "This script must be run under WeeChat."
    print "Get WeeChat now at: http://www.weechat.org/"
    quit()

# Script Properties
SCRIPT_NAME    = "auto_away"
SCRIPT_AUTHOR  = "Specimen"
SCRIPT_VERSION = "0.3.3"
SCRIPT_LICENSE = "GPL3"
SCRIPT_DESC    = "Simple auto-away script in Python"

# Default values
idletime = "20"
message = "Idle"

# Functions
def timer_hook_function():
    ''' Timer hook to check inactivity '''
    global timer_hook
    if val_idletime() > 0:
        timer_hook = w.hook_timer(10 * 1000, 60, 0, "idle_chk", "")
    return w.WEECHAT_RC_OK

def val_idletime():
    ''' Test idletime value '''
    try:
        idletime_value = int(w.config_get_plugin('idletime'))
    except ValueError:
        idletime_value = 0
    return idletime_value

def idle_chk(data, remaining_calls):
    ''' Inactivity check, when to change status to away '''
    global timer_hook
    if int(w.info_get("inactivity", "")) >= val_idletime() * 60:
        w.unhook(timer_hook)
        if not w.config_get_plugin('message'):
            w.config_set_plugin('message', message)
        w.command("", "/away -all %s" 
                  % w.config_get_plugin('message'))
        if int(version) < 0x00030200:
            ''' Workaround for /away -all bug in v. < 0.3.2 '''
            servers = irc_servers()
            if servers:
                for server in servers:
                    w.command(server, "/away %s" 
                              % w.config_get_plugin('message'))
        input_hook_function()
    return w.WEECHAT_RC_OK

def irc_servers():
    ''' Disconnected IRC servers, workaround for /away -all bug 
    in v. < 0.3.2 '''
    serverlist = w.infolist_get('irc_server','','')
    buffers = []
    if serverlist:
        while w.infolist_next(serverlist):
            if w.infolist_integer(serverlist, 'is_connected') == 0:
                buffers.append((w.infolist_pointer(serverlist,
                               'buffer')))
        w.infolist_free(serverlist)
    return buffers

def input_hook_function():
    ''' Input hook to check for typing '''
    global input_hook
    input_hook = w.hook_signal("input_text_changed",
                               "typing_chk", "")
    return w.WEECHAT_RC_OK

def typing_chk(data, signal, signal_data):
    ''' Activity check, when to return from away '''
    global input_hook
    w.unhook(input_hook)
    w.command("", "/away -all")
    if int(version) < 0x00030200:
        ''' Workaround for /away -all bug in v. < 0.3.2 '''
        servers = irc_servers()
        if servers:
            for server in servers:
                w.command(server, "/away")
    timer_hook_function()
    return w.WEECHAT_RC_OK

# Command hook and config hook
def autoaway_cmd(data, buffer, args):
    ''' /autoaway command, what to do with the arguments '''
    if args:
        value = args.strip(' ').partition(' ')
        w.config_set_plugin('idletime', value[0])
        if value[2]:
            w.config_set_plugin('message', value[2])
    if val_idletime() > 0:
        w.prnt(w.current_buffer(), 
               "%sauto-away%s settings:\n"
               "   Time:    %s%s%s minute(s)\n"
               "   Message: %s%s\n"
               % (w.color("bold"), w.color("-bold"),
               w.color("bold"), w.config_get_plugin('idletime'), 
               w.color("-bold"), w.color("bold"), 
               w.config_get_plugin('message')))
    else:
        w.prnt(w.current_buffer(), 
               "%sauto-away%s is disabled.\n"
               % (w.color("bold"), w.color("-bold")))
    return w.WEECHAT_RC_OK

def switch_chk(data, option, value):
    ''' Checks when idletime setting is changed '''
    global timer_hook, input_hook
    if timer_hook:
        w.unhook(timer_hook)
    if input_hook:
        w.unhook(input_hook)
    timer_hook_function()
    return w.WEECHAT_RC_OK

# Main
if __name__ == "__main__":
    if w.register(SCRIPT_NAME, SCRIPT_AUTHOR, SCRIPT_VERSION,
                  SCRIPT_LICENSE, SCRIPT_DESC, "", ""):
                  
        if not w.config_get_plugin('idletime'): 
            w.config_set_plugin('idletime', idletime)	
        if not w.config_get_plugin('message'): 
            w.config_set_plugin('message', message)
            
        w.hook_command("autoaway", 
                       "Set away status automatically after a period of "
                       "inactivity.", 
                       "[time|off] [message]", 
                       "      time: minutes of inactivity to set away\n"
                       "       off: disable auto-away (0 also disables)\n"
                       "   message: away message (optional)\n"
                       "\n"
                       "Without any arguments prints the current settings.\n"
                       "\n"
                       "Examples:\n"
                       "\n"
                       "/autoaway 20 I'm away\n"
                       "Sets auto-away to 20 minutes, and away message to "
                       "'I'm away'.\n"
                       "\n"
                       "/autoaway 30\n"
                       "Sets auto-away to 30 minutes, and uses the previously "
                       "set, or default, away message.\n"
                       "\n"
                       "/autoaway off\n"
                       "/autoaway 0\n"
                       "Disables auto-away.\n",
                       "", 
                       "autoaway_cmd", "")
        w.hook_config("plugins.var.python.auto_away.idletime",
                      "switch_chk", "")
                      
        version = w.info_get("version_number", "") or 0
        timer_hook = None
        input_hook = None

        timer_hook_function()



# -*- coding: utf-8 -*-
###
# Copyright (c) 2011 by Elián Hanisch <lambdae2@gmail.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
###

###
#   View and use WeeChat's infos.
#
#   Commands:
#   * /infos
#
#   History:
#     2013-01-06, Sebastien Helleu <flashcode@flashtux.org>:
#       version 0.2: make script compatible with Python 3.x
#     2011-10-02
#       version 0.1: new script!
#
###

try:
    import weechat
    from weechat import WEECHAT_RC_OK, prnt
    import_ok = True
except ImportError:
    print("This script must be run under WeeChat.")
    print("Get WeeChat now at: http://www.weechat.org/")
    import_ok = False

SCRIPT_NAME    = "infos"
SCRIPT_AUTHOR  = "Elián Hanisch <lambdae2@gmail.com>"
SCRIPT_VERSION = "0.2"
SCRIPT_LICENSE = "GPL3"
SCRIPT_DESC    = "View and use WeeChat's infos."

# -------------------------------------------------------------------------
# Class definitions

class Infolist(object):
    """Class for reading WeeChat's infolists."""

    fields = {
            'info_name'   :'string',
            'plugin_name' :'string',
            'description_nls' :'string',
            'args_description_nls' :'string',
            }

    def __init__(self, name, args=''):
        self.cursor = 0
        #debug('Generating infolist %r %r', name, args)
        self.pointer = weechat.infolist_get(name, '', args)
        if self.pointer == '':
            raise Exception("Infolist initialising failed (name:'%s' args:'%s')" %(name, args))

    def __len__(self):
        """True False evaluation."""
        if self.pointer:
            return 1
        else:
            return 0

    def __del__(self):
        """Purge infolist if is no longer referenced."""
        self.free()

    def __getitem__(self, name):
        """Implement the evaluation of self[name]."""
        value = getattr(weechat, 'infolist_%s' % self.fields[name])(self.pointer, name)
        return value

    def __iter__(self):
        def generator():
            while self.next():
                yield self
        return generator()

    def next(self):
        self.cursor = weechat.infolist_next(self.pointer)
        return self.cursor

    def prev(self):
        self.cursor = weechat.infolist_prev(self.pointer)
        return self.cursor

    def reset(self):
        """Moves cursor to beginning of infolist."""
        if self.cursor == 1: # only if we aren't in the beginning already
            while self.prev():
                pass

    def free(self):
        if self.pointer:
            #debug('Freeing Infolist')
            weechat.infolist_free(self.pointer)
            self.pointer = ''

# -------------------------------------------------------------------------
# Functions

def catchExceptions(f):
    def function(*args, **kwargs):
        try:
            return f(*args, **kwargs)
        except Exception as e:
            error(e)
    return function

script_nick = SCRIPT_NAME
def error(s, buffer=''):
    """Error msg"""
    prnt(buffer, '%s%s %s' % (weechat.prefix('error'), script_nick, s))
    if weechat.config_get_plugin('debug'):
        import traceback
        if traceback.sys.exc_type:
            trace = traceback.format_exc()
            prnt('', trace)

# -------------------------------------------------------------------------
# infos functions

def get_infos_list():
    return [ info['info_name'] for info in Infolist('hook', 'info') ]

def print_infos_description(buffer='', info_name=None):
    def print_desc(infolist):
        name = "%s[%s%s%s]" % (COLOR_CHAT_DELIMITERS,
                               COLOR_CHAT_BUFFER,
                               infolist['info_name'],
                               COLOR_CHAT_DELIMITERS)
        prnt(buffer, '')
        prnt(buffer, name)
        prnt(buffer, "plugin ........: %s%s" % (COLOR_CYAN,
                                                infolist['plugin_name'] or 'core'))
        prnt(buffer, "arguments .....: %s%s" % (COLOR_CYAN,
                                                infolist['args_description_nls']
                                                or '(no description)'))
        prnt(buffer, "description ...: %s%s" % (COLOR_CYAN,
                                                infolist['description_nls']
                                                or "(no description)"))

    infolist = Infolist('hook', 'info')
    found = False
    for info in infolist:
        if not info_name:
            print_desc(infolist)
        elif info_name == info['info_name']:
            found = True
            print_desc(infolist)

    if info_name and not found:
        prnt(buffer, "No info found with name \"%s\"" % info_name)

# -------------------------------------------------------------------------
# callbacks

@catchExceptions
def cmd_infos(data, buffer, args):
    cmd, _, args = args.partition(' ')
    info, _, args = args.partition(' ')
    if cmd == 'get' and info:
        if info not in get_infos_list():
            prnt('', "No info found with name \"%s\"" % info)
            return WEECHAT_RC_OK

        rt = weechat.info_get(info, args)
        header = "%s[%s%s%s] (%s%r%s)" % (COLOR_CHAT_DELIMITERS,
                                          COLOR_CHAT_BUFFER,
                                          info,
                                          COLOR_CHAT_DELIMITERS,
                                          COLOR_RESET,
                                          args,
                                          COLOR_CHAT_DELIMITERS)
        prnt('', '')
        prnt('', header)
        prnt('', "Result: %s%r" % (COLOR_CYAN, rt))
    elif cmd == 'show':
        print_infos_description(info_name=info)
    else:
        weechat.command('', "/help infos")

    return WEECHAT_RC_OK


def cmpl_infos_list(data, completion_item, buffer, completion):
    for name in get_infos_list():
        weechat.hook_completion_list_add(completion, name, 0, weechat.WEECHAT_LIST_POS_END)
    return WEECHAT_RC_OK

# -------------------------------------------------------------------------
# Main

if __name__ == '__main__' and import_ok and \
            weechat.register(SCRIPT_NAME, SCRIPT_AUTHOR,
                             SCRIPT_VERSION, SCRIPT_LICENSE,
                             SCRIPT_DESC, '', ''):

    # colors
    COLOR_RESET           = weechat.color('reset')
    COLOR_CHAT_DELIMITERS = weechat.color('chat_delimiters')
    COLOR_CHAT_NICK       = weechat.color('chat_nick')
    COLOR_CHAT_BUFFER     = weechat.color('chat_buffer')
    COLOR_CYAN            = weechat.color('cyan')


    # pretty [SCRIPT_NAME]
    script_nick = '%s[%s%s%s]%s' % (COLOR_CHAT_DELIMITERS,
                                    COLOR_CHAT_NICK,
                                    SCRIPT_NAME,
                                    COLOR_CHAT_DELIMITERS,
                                    COLOR_RESET)

    weechat.hook_command("infos", "View and use WeeChat infos",
                         "show [<info_name>] || get <info_name> [<arguments>]",
                         "show: Shows information about all infos or info <info_name>.\n"\
                         " get: Get info <info_name>.",
                         "get|show %(infos_info_list)",
                         "cmd_infos", "")

    weechat.hook_completion('infos_info_list', 'List of info names',
                            'cmpl_infos_list', '')

    # -------------------------------------------------------------------------
    # Debug

    if weechat.config_get_plugin('debug'):
        try:
            # custom debug module I use, allows me to inspect script's objects.
            import pybuffer
            debug = pybuffer.debugBuffer(globals(), '%s_debug' % SCRIPT_NAME)
        except ImportError:
            def debug(s, *args):
                if not isinstance(s, str):
                    s = str(s)
                if args:
                    s = s %args
                prnt('', '%s\t%s' % (script_nick, s))
    else:
        def debug(s, *args):
            pass


# vim:set shiftwidth=4 tabstop=4 softtabstop=4 expandtab textwidth=100:



# -*- coding: utf-8 -*-
######################################################################
# Copyright (c) 2011 by Pascal Wittmann <mail@pascal-wittmann.de>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# Marked Parts are from Wojciech Muła <wojciech_mula@poczta.onet.pl>
# and are licensed under BSD and are avaliable at
# http://0x80.pl/proj/aspell-python/
########################################################################

# INSTALLTION
# After copying this file into your python plugin directory, start weechat
# load the script and follow futher instructions calling
#    /help correction_completion
# You can find these instructions as markdown on
#    https://github.com/pSub/weechat-correction-completion/blob/master/README.md
# too.

# Changelog:
#
# 2012-10-18 -- Version 0.3.0
#    - incorrect words have a higher priority (typo completion)
#    - the following characters are removed from the typo: , . ; : ? ! ) ( \ / " ^
#
# 2011-06-01 -- Version 0.2.2
#    - fixed a memory leak, thanks to FlashCode
#    - documentation updates
#
# 2011-03-19 -- Version 0.2.1
#    - fixed bug that resulted in a crash of weechat
#
# 2011-03-18 -- Version 0.2.0
#    - fixed out of bounds bug
#    - apply config changes without reloading
#    - improved performance
#
# 2011-02-19 -- Version 0.1.0
#    - frist working version

import re

try:
    import ctypes
    import ctypes.util
except ImportError:
    print "This script depends on ctypes"

try:
    import weechat as w
except ImportError:
    print "This script must be run under WeeChat."
    print "Get WeeChat now at: http://www.weechat.org/"

SCRIPT_NAME    = "correction_completion"
SCRIPT_AUTHOR  = "Pascal Wittmann <mail@pascal-wittmann.de>"
SCRIPT_VERSION = "0.3.0"
SCRIPT_LICENSE = "GPL3"
SCRIPT_DESC    = "Provides a completion for 's/typo/correct'"
SCRIPT_COMMAND = "correction_completion"

# Default Options
# Your can use all aspell options listed on
# http://aspell.net/man-html/The-Options.html
settings = {
        'lang' : 'en',
}

# The Bunch Class is from
# http://code.activestate.com/recipes/52308/
class Bunch:
    def __init__(self, **kwds):
        self.__dict__.update(kwds)

def completion(data, completion_item, buffer, completion):
    if state.used == True:
        return w.WEECHAT_RC_OK
    else:
        state.used = True

    # Current cursor position
    pos = w.buffer_get_integer(buffer, 'input_pos')

    # Current input string
    input = w.buffer_get_string(buffer, 'input')

    fst = input.find("s/")
    snd = input.find("/", fst + 2)

    # Check for typo or suggestion completion
    if pos > 2 and fst >= 0 and fst < pos:
        if snd >= 0 and snd < pos:
          complete_replacement(pos, input, buffer)
        else:
          complete_typo(pos, input, buffer)

    state.used = False
    return w.WEECHAT_RC_OK

def complete_typo(pos, input, buffer):
    # Assume that typo changes when doing a completion
    state.curRepl = -1

    # Get the text of the current buffer
    list = []
    infolist = w.infolist_get('buffer_lines', buffer, '')
    while w.infolist_next(infolist):
        list.append(strip_symbols(w.infolist_string(infolist, 'message')))
    w.infolist_free(infolist)

    # Generate a list of words
    text = (' '.join(list)).split(' ')

    # Remove duplicate elements
    text = unify(text)

    # Split words in correct and incorrect ones
    good = [word for word in text if spellcheck(word) == True]
    bad  = [word for word in text if spellcheck(word) == False]

    # Sort by alphabet and length
    good.sort(key=lambda item: (item, len(item)))
    bad.sort(key=lambda item: (item, len(item)))

    # Place incorrcet ones in front of correct ones
    text = bad + good

    i = iter(text)

    # Get index of last occurence of "s/" befor cursor position
    n = input.rfind("s/", 0, pos)

    # Get substring and search the replacement
    substr = input[n+2:pos]
    replace = search((lambda word : word.startswith(substr)), i)

    # If no replacement found, display substring
    if replace == "":
      replace = substr

    # If substring perfectly matched take next replacement
    if replace == substr:
      try:
        replace = next(i)
      except StopIteration:
        replace = substr

    changeInput(substr, replace, input, pos, buffer)

def complete_replacement(pos, input, buffer):
    # Start Positions
    n = input.rfind("s/", 0, pos)
    m = input.rfind("/", n + 2, pos)

    repl = input[m + 1 : pos]
    typo = input[n + 2 : m]

    # Only query new suggestions, when typo changed
    if state.curRepl == -1 or typo != state.curTypo:
      state.suggestions = suggest(typo)
      state.curTypo = typo

    if len(state.suggestions) == 0:
      return

    # Start at begining when reached end of suggestions
    if state.curRepl == len(state.suggestions) - 1:
      state.curRepl = -1

    # Take next suggestion
    state.curRepl += 1

    # Put suggestion into the input
    changeInput(repl, state.suggestions[state.curRepl], input, pos, buffer)

def changeInput(search, replace, input, pos, buffer):
    # Put the replacement into the input
    n = len(search)
    input = '%s%s%s' %(input[:pos-n], replace, input[pos:])
    w.buffer_set(buffer, 'input', input)
    w.buffer_set(buffer, 'input_pos', str(pos - n + len(replace)))

def strip_symbols(string):
    return re_remove_chars.sub('', w.string_remove_color(string, ''))

def search(p, i):
    # Search for item matching the predicate p
    while True:
      try:
        item = next(i)
        if p(item):
          return item
      except StopIteration:
        return ""

def unify(list):
    # Remove duplicate elements from a list
    checked = []
    for e in list:
      if e not in checked:
        checked.append(e)
    return checked

# Parts are from Wojciech Muła
def suggest(word):
    if type(word) is str:
      suggestions = aspell.aspell_speller_suggest(
                      speller,
                      word.encode(),
                      len(word))
      elements = aspell.aspell_word_list_elements(suggestions)
      list = []
      while True:
          wordptr = aspell.aspell_string_enumeration_next(elements)
          if not wordptr:
              break;
          else:
              word = ctypes.c_char_p(wordptr)
              list.append(str(word.value))
      aspell.delete_aspell_string_enumeration(elements)
      return list
    else:
      raise TypeError("String expected")

def spellcheck(word):
    if type(word) is str:
        return aspell.aspell_speller_check(
            speller,
            word,
            len(word))
    else:
        raise TypeError("String expected")

def load_config(data = "", option = "", value = ""):
    global speller
    config = aspell.new_aspell_config()

    for option, default in settings.iteritems():
        if not w.config_is_set_plugin(option):
          w.config_set_plugin(option, default)
        value = w.config_get_plugin(option)
        if not aspell.aspell_config_replace(
                        config,
                        option.encode(),
                        value.encode()):
          raise Exception("Failed to replace config entry")

    # Error checking is from Wojciech Muła
    possible_error = aspell.new_aspell_speller(config)
    aspell.delete_aspell_config(config)
    if aspell.aspell_error_number(possible_error) != 0:
      aspell.delete_aspell_can_have_error(possible_error)
      raise Exception("Couldn't create speller")
    speller = aspell.to_aspell_speller(possible_error)
    return w.WEECHAT_RC_OK

if w.register(SCRIPT_NAME, SCRIPT_AUTHOR, SCRIPT_VERSION, SCRIPT_LICENSE, SCRIPT_DESC, "", ""):
    # Saving the current completion state
    state = Bunch(used = False, curTypo = '', curRepl = -1, suggestions = [])

    # Use ctypes to access the apsell library
    aspell = ctypes.CDLL(ctypes.util.find_library('aspell'))
    speller = 0

    # Regex to remove unwanted characters
    re_remove_chars = re.compile('[,.;:?!\)\(\\\/\"\^]')

    # Load configuration
    load_config()

    template = 'correction_completion'

    # Register completion hook
    w.hook_completion(template, "Completes after 's/' with words from buffer",
            'completion', '')

    # Register hook to update config when option is changed with /set
    w.hook_config("plugins.var.python." + SCRIPT_NAME + ".*", "load_config", "")

    # Register help command
    w.hook_command(SCRIPT_COMMAND, SCRIPT_DESC, "",
"""Usage:
If you want to correct yourself, you often do this using the
expression 's/typo/correct'. This plugin allows you to complete the
first part (the typo) by pressing *Tab*. The words from the actual
buffer are used to complet this part. If the word can be perfectly
matched the next word in alphabetical order is shown.

The second part (the correction) can also be completed. Just press
*Tab* after the slash and the best correction for the typo is fetched from aspell.
If you press *Tab* again, it shows the next suggestion.
The language used for suggestions can be set with the option

  plugins.var.python.correction_completion.lang

The aspell language pack must be installed for this language.

Setup:
Add the template %%(%(completion)s) to the default completion template.
The best way to set the template is to use the iset-plugin¹, because you can see
there the current value before changing it. Of course you can also use the
standard /set-command e.g.

  /set weechat.completion.default_template "%%(nicks)|%%(irc_channels)|%%(%(completion)s)"

Footnotes:
¹ http://weechat.org/scripts/source/stable/iset.pl/
"""
%dict(completion=template), '', '', '')



# -*- coding: utf-8 -*-
#
# Insert a giphy URL based on a command and search
# Use giphys random, search and translate from weechat
# Usage: /giphy search Search Term
# Usage: /giphy msg message
# Usage: /giphy random Search Term
# Usage: /gipgy Search Term
#
# History:
#
# 2017-04-19, butlerx
#   Version 1.0.1: remove + from message
# 2017-04-18, butlerx
#   Version 1.0.0: initial version
#

import requests
import weechat

SCRIPT_NAME = "giphy"
SCRIPT_AUTHOR = "butlerx <butlerx@redbrick.dcu.ie>"
SCRIPT_VERSION = "1.0.1"
SCRIPT_LICENSE = "GPL3"
SCRIPT_DESC = "Insert giphy gif"

URL = "http://api.giphy.com/v1/gifs/"
API = "&api_key=dc6zaTOxFJmzC"
RANDOM = "random?tag=%s"
TRANSLATE = "translate?s=%s"
SEARCH = "search?limit=1&q=%s"


def giphy(data, buf, args):
    """ Parse args to decide what api to use """
    search_string = args.split()
    arg = search_string.pop(0)
    search_string = "+".join(search_string)
    if arg == "search":
        image_url = search(URL + SEARCH + API, search_string)
    elif arg == "msg":
        image_url = translate(URL + TRANSLATE + API, search_string)
    elif arg == "random":
        image_url = random(URL + RANDOM + API, search_string)
    else:
        search_string = arg + "+" + search_string
        image_url = random(URL + RANDOM + API, search_string)
    weechat.command(buf, "giphy %s -- %s" %
                    (search_string.replace("+", " ").strip(), image_url))
    return weechat.WEECHAT_RC_OK


def translate(api, search_term):
    """Query giphy translate api for search"""
    response = requests.get(api % search_term)
    data = response.json()
    try:
        # Translate
        image_url = data["data"]["images"]["original"]["url"]
    except TypeError:
        image_url = "No GIF good enough"
    return image_url


def random(api, search_term):
    """Query giphy random api for search"""
    response = requests.get(api % search_term)
    data = response.json()
    try:
        # Random
        image_url = data["data"]["image_url"]
    except TypeError:
        image_url = "No GIF good enough"
    return image_url


def search(api, search_term):
    """Query giphy search api for search"""
    response = requests.get(api % search_term)
    data = response.json()
    try:
        image_url = data["data"][0]["images"]["original"]["url"]
    except TypeError:
        image_url = "No GIF good enough"
    return image_url


if __name__ == "__main__":
    if weechat.register(SCRIPT_NAME, SCRIPT_AUTHOR, SCRIPT_VERSION,
                        SCRIPT_LICENSE, SCRIPT_DESC, "", ""):
        weechat.hook_command("giphy", "Insert a giphy GIF", "",
                             "", "", "giphy", "")



# -*- coding: utf-8 -*-
#
# irssi_awaylog.py: emulates irssi awaylog (replay of hilights and privmsg)
# - 2013, henrik <henrik at affekt dot org>
#
# TODO: store awaylog in a file instead of memory
#
###########################################################################
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
###########################################################################

import_ok = True
try:
	import weechat as wc
except Exception:
	print "This script must be run under WeeChat."
	print "Get WeeChat now at: http://www.weechat.org/"
	import_ok = False

import time

SCRIPT_NAME     = "irssi_awaylog"
SCRIPT_AUTHOR   = "henrik"
SCRIPT_VERSION  = "0.3"
SCRIPT_LICENSE  = "GPL3"
SCRIPT_DESC     = "Emulates irssis awaylog behaviour"

awaylog = []

def replaylog():
	global awaylog

	if awaylog:
		wc.prnt("", "-->\t")
		for a in awaylog:
			wc.prnt_date_tags("", a[0], "", a[1])
		wc.prnt("", "<--\t")

		awaylog = []


def away_cb(data, bufferp, command):
	isaway = wc.buffer_get_string(bufferp, "localvar_away") != ""

	if not isaway:
		replaylog()
	return wc.WEECHAT_RC_OK

def msg_cb(data, bufferp, date, tagsn, isdisplayed, ishilight, prefix, message):
	global awaylog

	isaway = wc.buffer_get_string(bufferp, "localvar_away") != ""
	isprivate = wc.buffer_get_string(bufferp, "localvar_type") == "private"

	# catch private messages or highlights when away
	if isaway and (isprivate or int(ishilight)):
		logentry = "awaylog\t"

		if int(ishilight) and not isprivate:
			buffer = (wc.buffer_get_string(bufferp, "short_name") or
					wc.buffer_get_string(bufferp, "name"))
		else:
			buffer = "priv"

		buffer = wc.color("green") + buffer + wc.color("reset")

		logentry += "[" + buffer + "]"
		logentry += wc.color("default") + " <" + wc.color("blue") + prefix + wc.color("default") + "> " + wc.color("reset") + message

		awaylog.append((int(time.time()), logentry))
	return wc.WEECHAT_RC_OK

if __name__ == "__main__":
	if import_ok and wc.register(SCRIPT_NAME, SCRIPT_AUTHOR, SCRIPT_VERSION, SCRIPT_LICENSE, SCRIPT_DESC, "", ""):
		wc.hook_print("", "notify_message", "", 1, "msg_cb", "")
		wc.hook_print("", "notify_private", "", 1, "msg_cb", "")
		wc.hook_command_run("/away", "away_cb", "")



# coding: utf-8
#
# Copyright (c) 2012 by nesthib <nesthib@gmail.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
# This script checks every message before it is sent and blocks messages which
# correspond to misformatted commands (e.g. " /msg NickServ…") to avoid the
# unfortunate discosure of personnal informations.
#
# 2012-03-07: nesthib <nesthib@gmail.com>
#        0.1: initial release

try:
    import weechat as w
except Exception:
    print("This script must be run under WeeChat.")
    print("Get WeeChat now at: http://www.weechat.org/")
    quit()

import re

name = "unwanted_msg"
author = "nesthib <nesthib@gmail.com>"
version = "0.1"
license = "GPL"
description = "Avoid sending misformatted commands as messages"
shutdown_function = ""
charset = ""

w.register(name, author, version, license, description, shutdown_function, charset)

settings = {
        'regexp'        : ' +/',     # if the pattern matches the beginning of the line, the message will be blocked
        'warning_buffer': 'current', # if set to current/server/weechat will print warning on current/server/weechat buffer. Disable warning if unset
        }
for opt, val in settings.iteritems():
    if not w.config_is_set_plugin(opt):
        w.config_set_plugin(opt, val)

def my_modifier_cb(data, modifier, modifier_data, string):
    if unwanted_pattern.match(string):
        if options['warning_buffer'] == 'current':
            output = w.current_buffer()
        elif options['warning_buffer'] == 'server':
            server = w.buffer_get_string(w.current_buffer(), 'localvar_server')
            plugin = w.buffer_get_string(w.current_buffer(), 'plugin')
            output = w.buffer_search(plugin, 'server.'+server)
        elif options['warning_buffer'] == '':
            output = None
        else:
            output = '' # if invalid option set to weechat buffer
        if not output == None:
            w.prnt_date_tags(output, 0, 'no_log', '%sunwanted message deleted: "%s"' % (w.prefix('error'), string))
        w.buffer_set(w.current_buffer(), 'input', string)
        return ''
    else:
        return string

options = {}
for option in settings.keys():
    options[option] = w.config_get_plugin(option)
unwanted_pattern = re.compile(options['regexp'])

def my_config_cb(data, option, value):
    global options, unwanted_pattern
    for option in settings.keys():
        options[option] = w.config_get_plugin(option)
    unwanted_pattern = re.compile(options['regexp'])
    return w.WEECHAT_RC_OK

for option in settings.keys():
    w.hook_config("plugins.var.python.%s.%s" % (name, option), "my_config_cb", "")

w.hook_modifier('input_text_for_buffer', 'my_modifier_cb', '')



# -*- coding: utf-8 -*-
#
# Copyright (c) 2015 by nils_2 <weechatter@arcor.de>
#
# mark buffer as unread
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# idea by Phyks
#
# 2015-03-13: nils_2, (freenode.#weechat)
#       1 : initial release

try:
    import weechat
    import time

except Exception:
    print("This script must be run under WeeChat.")
    print("Get WeeChat now at: http://www.weechat.org/")
    quit()

SCRIPT_NAME     = "unread_buffer"
SCRIPT_AUTHOR   = "nils_2 <weechatter@arcor.de>"
SCRIPT_VERSION  = "1"
SCRIPT_LICENSE  = "GPL"
SCRIPT_DESC     = "mark buffer as unread"

def unread_buffer_cb(data, buffer, args):

    arguments = args.lower().split(' ')
    if not len(arguments) == 2:
        return weechat.WEECHAT_RC_OK

    if arguments[0] == 'low':
        priority = "0"
    elif arguments[0] == 'message':
        priority = "1"
    elif arguments[0] == 'private':
        priority = "2"
    elif arguments[0] == 'highlight':
        priority = "3"
    else:
        return weechat.WEECHAT_RC_OK
    # search for either buffer_name or buffer_number
    ptr_buffer = buffer_infolist(arguments[1])

    # check if buffer is *not* in hotlist
    if not check_hotlist(ptr_buffer):
        weechat.buffer_set(ptr_buffer, "hotlist", priority)

    return weechat.WEECHAT_RC_OK

def buffer_infolist(buf_name):
    ptr_buffer = 0
    infolist = weechat.infolist_get('buffer', '', '')
    while weechat.infolist_next(infolist):
        short_name = weechat.infolist_string(infolist, 'short_name')
        name = weechat.infolist_string(infolist, 'name')
        number = weechat.infolist_integer(infolist, 'number')
        matching = name.lower().find(buf_name) >= 0
        if matching:
            ptr_buffer = weechat.infolist_pointer(infolist, 'pointer')
            break
        if not matching and buf_name.isdigit():
            matching = str(number).startswith(buf_name)
            if len(buf_name) == 0 or matching:
                ptr_buffer = weechat.infolist_pointer(infolist, 'pointer')
                break
    weechat.infolist_free(infolist)
    if ptr_buffer:
        buf_type = weechat.buffer_get_string(ptr_buffer, "localvar_type")
        # buffer has no type (e.g. channel, private, server, weechat)
        if buf_type == '':
            ptr_buffer = 0
    return ptr_buffer

def check_hotlist(check_pointer):
    ptr_buffer = 0
    infolist = weechat.infolist_get('hotlist', '', '')
    while weechat.infolist_next(infolist):
        pointer = weechat.infolist_pointer(infolist, 'buffer_pointer')
        if pointer == check_pointer:
            ptr_buffer = pointer
            break
    weechat.infolist_free(infolist)
    return ptr_buffer
# ================================[ main ]===============================
if __name__ == "__main__":
    if weechat.register(SCRIPT_NAME, SCRIPT_AUTHOR, SCRIPT_VERSION, SCRIPT_LICENSE, SCRIPT_DESC, '', ''):
        WEECHAT_VERSION = weechat.info_get("version_number", "") or 0

        if int(WEECHAT_VERSION) >= 0x01000000:
            weechat.hook_command(SCRIPT_NAME, SCRIPT_DESC, 'low <buffer>||message <buffer>||private <buffer>||highlight <buffer>',
                                 'Notify levels:\n'
                                 '  low      : message with low importance (for example irc join/part/quit)\n'
                                 '  message  : message from a user\n'
                                 '  private  : message in a private buffer\n'
                                 '  highlight: message with highlight\n'
                                 '\n'
                                 '<buffer> can be a buffer name or a buffer number\n'
                                 '\n'
                                 'Example:\n'
                                 '  /unread_buffer highlight 3\n'
                                 '  /unread_buffer message freenode.#weechat\n',
                                 'low|message|private|highlight %(buffers_names)|%(buffers_numbers) %-',
                                 'unread_buffer_cb',
                                 '')
        else:
            weechat.prnt('','%s%s %s' % (weechat.prefix('error'),SCRIPT_NAME,': needs version 1.0 or higher'))



# -*- coding: utf-8 -*-
#
# buffer_swap, version 0.3 for WeeChat version 0.3
# Latest development version: https://github.com/FiXato/weechat_scripts
#
#   Swaps given 2 buffers. Requested by kakazza
#
## Example:
#  Swaps buffers 3 and 5
#   /swap 3 5
#
#  Swaps current buffer with the #weechat buffer
#   /swap #weechat
#
## History:
### 2011-09-18: FiXato:
# 
# * version 0.1: initial release.
#     * Allow switching 2 given buffers
#
# * version 0.2: cleanup
#     * Made the command example more clear that it requires 2 buffer *numbers*
#     * After switching, now switches back to your original buffer.
#
# * version 0.3: current buffer support
#     * If you only specify 1 buffer, the current buffer will be used
#
## Acknowledgements:
# * Sebastien "Flashcode" Helleu, for developing the kick-ass chat/IRC
#    client WeeChat
#
## TODO: 
#   - Check if given buffers exist.
#
## Copyright (c) 2011 Filip H.F. "FiXato" Slagter,
#   <FiXato [at] Gmail [dot] com>
#   http://google.com/profiles/FiXato
#
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the Software, and to
# permit persons to whom the Software is furnished to do so, subject to
# the following conditions:
#
# The above copyright notice and this permission notice shall be
# included in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
# LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
# OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
# WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
#
SCRIPT_NAME     = "buffer_swap"
SCRIPT_AUTHOR   = "Filip H.F. 'FiXato' Slagter <fixato [at] gmail [dot] com>"
SCRIPT_VERSION  = "0.3"
SCRIPT_LICENSE  = "MIT"
SCRIPT_DESC     = "Swaps given 2 buffers"
SCRIPT_COMMAND  = "swap"
SCRIPT_CLOSE_CB = "close_cb"

import_ok = True

try:
  import weechat
except ImportError:
  print "This script must be run under WeeChat."
  import_ok = False

def close_cb(*kwargs):
  return weechat.WEECHAT_RC_OK

def command_main(data, buffer, args):
  args = args.split()
  curr_buffer = weechat.current_buffer()
  curr_buffer_number = weechat.buffer_get_integer(curr_buffer, "number")

  if len(args) != 1 and len(args) != 2:
    weechat.prnt("", "You need to specify 1 or 2 buffers")
    return weechat.WEECHAT_RC_ERROR

  if len(args) == 2:
    weechat.command("", "/buffer %s" % args[0])
    first_buffer = weechat.current_buffer()
    first_buffer_number = weechat.buffer_get_integer(first_buffer, "number")

    weechat.command("", "/buffer %s" % args[1])
    second_buffer = weechat.current_buffer()
    second_buffer_number = weechat.buffer_get_integer(second_buffer, "number")
  else:
    first_buffer = weechat.current_buffer()
    first_buffer_number = weechat.buffer_get_integer(first_buffer, "number")

    weechat.command("", "/buffer %s" % args[0])
    second_buffer = weechat.current_buffer()
    second_buffer_number = weechat.buffer_get_integer(second_buffer, "number")
  
  weechat.buffer_set(first_buffer, "number", str(second_buffer_number))
  weechat.buffer_set(second_buffer, "number", str(first_buffer_number))

  weechat.command("", "/buffer %s" % str(curr_buffer_number))
  
  return weechat.WEECHAT_RC_OK

if __name__ == "__main__" and import_ok:
  if weechat.register(SCRIPT_NAME, SCRIPT_AUTHOR, SCRIPT_VERSION,
                      SCRIPT_LICENSE, SCRIPT_DESC, SCRIPT_CLOSE_CB, ""):
    #        # Set default settings
    #        for option, default_value in cs_settings.iteritems():
    #            if not weechat.config_is_set_plugin(option):
    #                weechat.config_set_plugin(option, default_value)

    weechat.hook_command(SCRIPT_COMMAND, 
                          SCRIPT_DESC,
                          "[buffer] <buffer to swap with>",
                          "Swaps given buffers: \n"
                          "the /swap command accepts anything that /buffer would accept for switching buffers\n"
                          "/swap 3 10\n"
                          "would swap buffer 3 and 10 of place\n"
                          "/swap 3\n"
                          "would swap current buffer with buffer number 10\n"
                          "/swap 3 #weechat\n"
                          "would swap buffer 3 and the #weechat buffer of place\n"
                          "/swap #weechat\n"
                          "would swap current buffer with the #weechat buffer",

                          "",

                          "command_main", "")





# -*- coding: utf-8 -*-
#
# Copyright (C) 2016 Jasper v. Blanckenburg <jasper@mezzo.de>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

#
# Play back all messages the ZNC bouncer received since you last received a
# message locally
# (this requires the Playback module to be loaded on ZNC)
#
# Add znc.in/playback either to irc.server_default.capabilites or
# to the servers you want to use the Playback module on, then set
# plugins.var.python.zncplayback.servers
#
# History:
#
# 2016-08-27, Jasper v. Blanckenburg <jasper@mezzo.de>:
#     v0.1.0: Initial release

SCRIPT_NAME = "zncplayback"
SCRIPT_AUTHOR = "jazzpi"
SCRIPT_VERSION = "0.1.0"
SCRIPT_LICENSE = "GPL3"
SCRIPT_DESCRIPTION = "Add support for the ZNC Playback module"

SCRIPT_SAVEFILE = "zncplayback_times.json"

import_ok = True

try:
    import weechat
except ImportError:
    print("This script must be run under WeeChat.")
    print("Get WeeChat now at: http://www.weechat.org/")
    import_ok = False

try:
    import json
    import os.path as path
    from time import time
except ImportError as message:
    print("Missing package(s) for {}: {}".format(SCRIPT_NAME, message))
    import_ok = False

# Script options
zncplayback_settings_default = {
    "servers": (
        "",
        "Comma-separated list of servers that playback should be fetched for")
}
zncplayback_settings = {}

zncplayback_hooks = {
    "config": None,
    "connected": None,
    "messages": {}
}
zncplayback_last_times = {}


def write_last_times():
    """Write the last message times of all servers to disk."""
    with open(SCRIPT_SAVEFILE, "w") as fh:
        json.dump(zncplayback_last_times, fh)


def read_last_times():
    """Read the last message times of all servers from disk."""
    global zncplayback_last_times
    if not path.exists(SCRIPT_SAVEFILE):
        for server in zncplayback_settings["servers"].split(","):
            zncplayback_last_times[server] = 0
        return
    with open(SCRIPT_SAVEFILE, "r") as fh:
        zncplayback_last_times = json.load(fh)


def zncplayback_config_cb(data, option, value):
    """Update in-memory settings when a script option is changed."""
    global zncplayback_settings
    pos = option.rfind(".")
    if pos > 0:
        name = option[pos+1:]
        if name in zncplayback_settings:
            if name == "servers":
                old_servers = zncplayback_settings["servers"].split(",")
                new_servers = value.split(",")
                # Unhook signals for old servers
                removed_servers = \
                    [s for s in old_servers if s not in new_servers]
                for serv in removed_servers:
                    weechat.unhook(zncplayback_hooks["messages"][serv])
                    del zncplayback_last_times[serv]
                # Hook signals for new servers
                added_servers = \
                    [s for s in new_servers if s not in old_servers]
                for serv in added_servers:
                    zncplayback_hooks["messages"][serv] = weechat.hook_signal(
                        "{},irc_raw_in_PRIVMSG".format(serv),
                        "zncplayback_message_cb", serv)
                    zncplayback_last_times[serv] = 0
            zncplayback_settings[name] = value
    return weechat.WEECHAT_RC_OK


def zncplayback_connected_cb(data, signal, server):
    """Fetch playback after connecting to a server."""
    if server not in zncplayback_settings["servers"].split(","):
        return weechat.WEECHAT_RC_OK
    buf = weechat.buffer_search("irc", "server.{}".format(server))
    weechat.command(buf,
        "/msg *playback PLAY * {}".format(zncplayback_last_times[server]))
    return weechat.WEECHAT_RC_OK


def zncplayback_message_cb(server, signal, message):
    """Update last time for a server when a PRIVMSG is sent."""
    global zncplayback_last_times
    if server not in zncplayback_settings["servers"].split(","):
        return weechat.WEECHAT_RC_OK
    zncplayback_last_times[server] = int(time())
    write_last_times()
    return weechat.WEECHAT_RC_OK


if __name__ == "__main__" and import_ok:
    if weechat.register(SCRIPT_NAME, SCRIPT_AUTHOR, SCRIPT_VERSION,
                        SCRIPT_LICENSE, SCRIPT_DESCRIPTION, "", ""):
        SCRIPT_SAVEFILE = path.join(weechat.info_get("weechat_dir", ""),
                                    SCRIPT_SAVEFILE)
        # Set default settings
        version = weechat.info_get("version_number", "") or 0
        for option, value in zncplayback_settings_default.items():
            if weechat.config_is_set_plugin(option):
                zncplayback_settings[option] = weechat.config_get_plugin(
                    option)
            else:
                weechat.config_set_plugin(option, value[0])
                zncplayback_settings[option] = value[0]
            if int(version) >= 0x00030500:
                weechat.config_set_desc_plugin(
                    option,
                    "%s (default: \"%s\")" % (value[1], value[0]))

        read_last_times()
        zncplayback_hooks["config"] = weechat.hook_config(
            "plugins.var.python.%s.*" % SCRIPT_NAME, "zncplayback_config_cb",
            "")
        zncplayback_hooks["connected"] = weechat.hook_signal(
            "irc_server_connected", "zncplayback_connected_cb", "")
        for serv in zncplayback_settings["servers"].split(","):
            zncplayback_hooks["messages"][serv] = weechat.hook_signal(
                "{},irc_raw_in_PRIVMSG".format(serv),
                "zncplayback_message_cb", serv)

        # TODO: Unhook when unloading script



# -*- coding: utf-8 -*-
#
# Copyright (C) 2013  Stefan Wold <ratler@stderr.eu>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# (This script requires WeeChat 0.3.0 or higher).
#
# WeeChat script that convert ascii emotes to the Unicode (version 6.3) equivalent emoticon.
#
# Source available on GitHUB: https://github.com/Ratler/ratlers-weechat-scripts
#
# Contributors:
# Nicolas G. Querol
#
# Commands:
# /weemoticons - List supported emoticons in the current buffer

SCRIPT_NAME    = "weemoticons"
SCRIPT_AUTHOR  = "Stefan Wold <ratler@stderr.eu>"
SCRIPT_VERSION = "0.2"
SCRIPT_LICENSE = "GPL3"
SCRIPT_DESC    = "Convert ascii emotes to unicode emoticons."
SCRIPT_COMMAND = "weemoticons"

import_ok = True

try:
    import weechat
    import re
except ImportError:
    print "This script must be run under WeeChat."
    import_ok = False

ICONS = {
    '^^': u'\U0001F601', '^_^': u'\U0001F601',  # GRINNING FACE WITH SMILING EYES
    # '': u'\U0001F602',  # FACE WITH TEARS OF JOY
    ':)': u'\U0001F603', ':-)': u'\U0001F603', '=)': u'\U0001F603',  # SMILING FACE WITH OPEN MOUTH
    ':D': u'\U0001F604', '=D': u'\U0001F604',  # SMILING FACE WITH OPEN MOUTH AND SMILING EYES
    # '': u'\U0001F605',  # SMILING FACE WITH OPEN MOUTH AND COLD SWEAT
    # '': u'\U0001F606',  # SMILING FACE WITH OPEN MOUTH AND TIGHTLY-CLOSED EYES
    # '': u'\U0001F607',  # SMILING FACE WITH HALO
    '>:D': u'\U0001F608', '>=D': u'\U0001F608',  # SMILING FACE WITH HORNS
    ';)': u'\U0001F609', ';-)': u'\U0001F609',  # WINKING FACE
    '8)': u'\U0001F60A', 'B)': u'\U0001F60A',   # SMILING FACE WITH SMILING EYES
    # '': u'\U0001F60B',  # FACE SAVOURING DELICIOUS FOOD
    # '': u'\U0001F60C',  # RELIEVED FACE
    # '': u'\U0001F60D',  # SMILING FACE WITH HEART-SHAPED EYES
    # '': u'\U0001F60E',  # SMILING FACE WITH SUNGLASSES
    # '': u'\U0001F60F',  # SMIRKING FACE
    # '': u'\U0001F610',  # NEUTRAL FACE
    # '': u'\U0001F611',  # EXPRESSIONLESS FACE
    ':|': u'\U0001F612', '=|': u'\U0001F612', '>_>': u'\U0001F612', '<_<': u'\U0001F612',  # UNAMUSED FACE
    # '': u'\U0001F613',  # FACE WITH COLD SWEAT
    # '': u'\U0001F614',  # PENSIVE FACE
    ':S': u'\U0001F615', ':/': u'\U0001F615', ':\\': u'\U0001F615', '=S': u'\U0001F615', '=/': u'\U0001F615',
    '=\\': u'\U0001F615',  # CONFUSED FACE
    # '': u'\U0001F616',  # CONFOUNDED FACE
    # '': u'\U0001F617',  # KISSING FACE
    # '': u'\U0001F618',  # FACE THROWING A KISS
    # '': u'\U0001F619',  # KISSING FACE WITH SMILING EYES
    # '': u'\U0001F61A',  # KISSING FACE WITH CLOSED EYES
    ':P': u'\U0001F61B', ':p': u'\U0001F61B', '=P': u'\U0001F61B', '=p': u'\U0001F61B',  # FACE WITH STUCK-OUT TONGUE
    ';P': u'\U0001F61C', ';-P': u'\U0001F61C',  # FACE WITH STUCK-OUT TONGUE AND WINKING EYE
    # '': u'\U0001F61D',  # FACE WITH STUCK-OUT TONGUE AND TIGHTLY-CLOSED EYES
    ':(': u'\U0001F61E', ':-(': u'\U0001F61E', '=(': u'\U0001F61E', '=-(': u'\U0001F61E',  # DISAPPOINTED FACE
    # '': u'\U0001F61F',  # WORRIED FACE
    # '': u'\U0001F620',  # ANGRY FACE
    '>:(': u'\U0001F621', '>=(': u'\U0001F621',  # POUTING FACE
    ':\'(': u'\U0001F622', '=\'(': u'\U0001F622',  # CRYING FACE
    '>_<': u'\U0001F623',  # PERSEVERING FACE
    # '': u'\U0001F624',  # FACE WITH LOOK OF TRIUMPH
    # '': u'\U0001F625',  # DISAPPOINTED BUT RELIEVED FACE
    # '': u'\U0001F626',  # FROWNING FACE WITH OPEN MOUTH
    # '': u'\U0001F627',  # ANGUISHED FACE
    # '': u'\U0001F628',  # FEARFUL FACE
    # '': u'\U0001F629',  # WEARY FACE
    # '': u'\U0001F62A',  # SLEEPY FACE
    # '': u'\U0001F62B',  # TIRED FACE
    # '': u'\U0001F62C',  # GRIMACING FACE
    # '': u'\U0001F62D',  # LOUDLY CRYING FACE
    # '': u'\U0001F62E',  # FACE WITH OPEN MOUTH
    # '': u'\U0001F62F',  # HUSHED FACE
    # '': u'\U0001F630',  # FACE WITH OPEN MOUTH AND COLD SWEAT
    # '': u'\U0001F631',  # FACE SCREAMING IN FEAR
    # '': u'\U0001F632',  # ASTONISHED FACE
    ':")': u'\U0001F633', '=")': u'\U0001F633',  # FLUSHED FACE
}

ICON_PATTERN = re.compile(r"(?<!\S)([>;:=8B\^]\S{1,2})")

def icon(match):
    global ICONS
    emoticon = match.group(0)

    if emoticon in ICONS:
        return "%s " % ICONS[emoticon].encode("utf-8")

    return emoticon

def convert_icon_cb(data, modifier, modifier_data, message):
    global ICON_PATTERN

    plugin, buf, tags = modifier_data.split(';')
    tags = tags.split(',')

    if 'irc_privmsg' in tags or 'irc_notice' in tags:
        if ICON_PATTERN.search(message):
            message = ICON_PATTERN.sub(icon, message)

    return message

def list_icons_cb(data, buf, args):
    global ICONS

    l = dict()
    for key, val in ICONS.items():
        if val in l:
            l[val] += ", " + key
        else:
            l[val] = key

    weechat.prnt(buf, "%s - list of supported emoticons:" % SCRIPT_NAME)
    [weechat.prnt(buf, " %s  = %s" % (key.encode("utf-8"), l[key])) for key in l.keys()]

    return weechat.WEECHAT_RC_OK

if __name__ == "__main__" and import_ok:
    if weechat.register(SCRIPT_NAME, SCRIPT_AUTHOR, SCRIPT_VERSION, SCRIPT_LICENSE, SCRIPT_DESC, "", ""):
        weechat.hook_modifier("weechat_print", "convert_icon_cb", "")
        weechat.hook_command(SCRIPT_COMMAND, "List supported emoticons", "", "", "", "list_icons_cb", "")



# -*- coding: utf-8 -*-
#
# Copyright (c) 2010 by Alexander Schremmer <alex@alexanderweb.de>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

#
# (this script requires WeeChat 0.3.0 or newer)
#
# History:
# 2015-04-29, Colgate Minuette <rabbit@minuette.net>
#   version 0.2.3: add option to send queued messages on /nick
# 2013-11-08, Stefan Huber <shuber@sthu.org>
#   version 0.2.2: add match_prefix setting, recall timestamp of message
# 2012-12-29, Stefan Huber <shuber@sthu.org>
#   version 0.2.1: fix channel determination in join_cb
# 2010-05-20, Alexander Schremmer <alex@alexanderweb.de>
#   version 0.2: removed InfoList code
# 2010-05-15, Alexander Schremmer <alex@alexanderweb.de>
#   version 0.1: initial release

import weechat as w
import re
from datetime import datetime
from time import strftime

SCRIPT_NAME    = "postpone"
SCRIPT_AUTHOR  = "Alexander Schremmer <alex@alexanderweb.de>"
SCRIPT_VERSION = "0.2.3"
SCRIPT_LICENSE = "GPL3"
SCRIPT_DESC    = "Postpones written messages for later dispatching if target nick is not on channel"

postpone_data = {}

settings = {
        'match_prefix': ('', 'Postpone message if prefix before "nick:" is matched. (Default: "")'),
        'message_on_nick': ('off', 'Send message on /nick in addition to /join (Default: off)')
}

def send_messages(server, channel, nick):
    buffer = w.buffer_search("", "%s.%s" % (server, channel))
    messages = postpone_data[server][channel][nick]
    for time, msg in messages:
        tstr = strftime("%Y-%m-%d %H:%M:%S", time.timetuple())
        w.command(buffer, msg + " (This message has been postponed on " + tstr + ".)")
    messages[:] = []

def join_cb(data, signal, signal_data):
    server = signal.split(',')[0] # EFNet,irc_in_JOIN
    channel = re.match('.* JOIN :?(?P<channel>.+)$', signal_data).groups()[0]
    nick = re.match(':(?P<nick>.+)!', signal_data).groups()[0].lower()
    buffer = w.buffer_search("", "%s.%s" % (server, channel))
    if server in postpone_data and channel in postpone_data[server] and\
            nick in postpone_data[server][channel]:
        send_messages(server, channel, nick)
    return w.WEECHAT_RC_OK

def nick_cb(data, signal, signal_data):

    if not w.config_is_set_plugin('message_on_nick'):
        return w.WEECHAT_RC_OK
    if not w.config_get_plugin('message_on_nick').lower() == "on":
        return w.WEECHAT_RC_OK

    server = signal.split(",")[0]
    if server in postpone_data:
        nick = signal_data.split(" ")[2]
        if nick.startswith(":"):
           nick = nick[1:]
        nick = nick.lower()
        for channel in postpone_data[server]:
            if nick in postpone_data[server][channel]:
                send_messages(server, channel, nick)
    return w.WEECHAT_RC_OK

def channel_has_nick(server, channel, nick):
    buffer = w.buffer_search("", "%s.%s" % (server, channel))
    return bool(w.nicklist_search_nick(buffer, "", nick))

def command_run_input(data, buffer, command):
    """ Function called when a command "/input xxxx" is run """
    if command == "/input return": # As in enter was pressed.
        input_s = w.buffer_get_string(buffer, 'input')
        server = w.buffer_get_string(buffer, 'localvar_server')
        channel = w.buffer_get_string(buffer, 'localvar_channel')
        match_prefix = w.config_get_plugin('match_prefix')

        match = re.match(match_prefix + r'([\w-]+?): (.*)$', input_s)
        if match:
            nick, message = match.groups()
            if not channel_has_nick(server, channel, nick):
                w.prnt(buffer, "| Enqueued message for %s: %s" % (nick, message))
                save = datetime.now(), nick + ": " + message
                postpone_data.setdefault(server, {}).setdefault(channel,
                        {}).setdefault(nick.lower(), []).append(save)
                w.buffer_set(buffer, 'input', "")
                # XXX why doesn't this work? i want to have the typed text
                # in the history
                #history_list = w.infolist_get("history", buffer, "")
                #history_item = w.infolist_new_item(history_list)
                #w.infolist_new_var_string(history_item, "text", input_s)
    return w.WEECHAT_RC_OK


if w.register(SCRIPT_NAME, SCRIPT_AUTHOR, SCRIPT_VERSION, SCRIPT_LICENSE,
                    SCRIPT_DESC, "", ""):

    version = w.info_get('version_number', '') or 0
    for option, default_desc in settings.iteritems():
        if not w.config_is_set_plugin(option):
            w.config_set_plugin(option, default_desc[0])
        if int(version) >= 0x00030500:
            w.config_set_desc_plugin(option, default_desc[1])

    w.hook_command_run("/input return", "command_run_input", "")
    w.hook_signal('*,irc_in2_join', 'join_cb', '')
    w.hook_signal('*,irc_in2_nick', 'nick_cb', '')



# -*- coding: utf-8 -*-
# zncnotice: weechat script to convert ZNC status PRIVMSGs to NOTICEs
# © 2016 Hugo Landau <hlandau@devever.net>  MIT License
# Tested with Weechat 1.5. Older versions may or may not work.
#
# Settings:
#   plugins.var.python.zncnotice.prefix=*
#     The nickname prefix used by ZNC for its psuedo-users. Usually * but can be
#     changed to something else. Must match the ZNC configuration. (default: "*")
#
# Instructions for use:
#   1. Load script.
#   2. If using a prefix other than *, set plugins.var.python.zncnotice.prefix.
#   3. PRIVMSGs from nicknames with that prefix will now be converted to NOTICEs.

SCRIPT_NAME = 'zncnotice'
SCRIPT_AUTHOR = 'hlandau'
SCRIPT_VERSION = '1.0.0'
SCRIPT_LICENSE = 'MIT'
SCRIPT_DESC = 'Convert privmsg to notice from nicks with a certain prefix (useful for ZNC)'

import weechat
import re

re_match = None

def set_re(prefix):
  global re_match
  if prefix == '':
    prefix = '*'
  re_match = re.compile(r'''^(:'''+re.escape(prefix)+r'''[^!]+![^ ]+) PRIVMSG ''')

def irc_in_privmsg(data, signal, server, args):
  return re_match.sub('\\1 NOTICE ', args)

def config_change(data, opt, value):
  set_re(value)
  return weechat.WEECHAT_RC_OK

if __name__ == '__main__':
  weechat.register(SCRIPT_NAME, SCRIPT_AUTHOR, SCRIPT_VERSION, SCRIPT_LICENSE, SCRIPT_DESC, '', '')

  settings = dict(prefix=['*','The nickname prefix used by ZNC for its psuedo-users. Usually * but can be changed to something else. Must match the ZNC configuration.'])
  for k, v in settings.items():
    if not weechat.config_is_set_plugin(k):
      weechat.config_set_plugin(k, v[0])

    weechat.config_set_desc_plugin(k, '%s (default: "%s")' % (v[1], v[0]))

  set_re(weechat.config_get_plugin('prefix'))
  weechat.hook_modifier('irc_in_privmsg', 'irc_in_privmsg', '')
  weechat.hook_config('plugins.var.python.zncnotice.prefix', 'config_change', '')



"""Set channel priority

Based on the idea of Flavius Aspra <flavius.as@gmail.com>

Author: Barbu Paul - Gheorghe <paullik.paul@gmail.com>

When joining channels the buffers will be arranged from high to low priority.
Buffers containing high-priority channels will be in the head of the list:
Positions: 2, 3, 4, ...
They will be followed by low (normal) priority channels.

High-priority channels must be set using this command:
    /chanpriority [#channels]+

The argument must be a list of channels separated by commas, e.g.:
    /chanpriority #chan1, #chan2, #chan3

Channels can also be set using the format: network.#channel:
    /chanpriority #chan1, freenode.#weechat, network.org.#channel

LICENSE:

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

import weechat

SCRIPT_COMMAND = "chanpriority"
SCRIPT_AUTHOR = "Barbu Paul - Gh. <paullik.paul@gmail.com>"
SCRIPT_VERSION = "0.5"
SCRIPT_NAME = "chanpriority"
SCRIPT_LICENSE = "GPL3"
SCRIPT_DESC = "Allows you to set high-priority channels, see /help chanpriority"

whitelist = []

def reorder_buffers():
    """Reorders the buffers once the whitelist has changed
    """

    count = 2
    chan = ""

    ilist = weechat.infolist_get("buffer", "", "")

    for chan in whitelist:
        if -1 == chan.find(".#"): #network name is not set, matching by short_name
            weechat.infolist_reset_item_cursor(ilist)

            while weechat.infolist_next(ilist):
                if weechat.infolist_string(ilist, "short_name") == chan:
                    chan = weechat.infolist_string(ilist, "name")
                    break

        buff = weechat.buffer_search("irc", chan)

        if buff:
            weechat.buffer_set(buff, "number", str(count))
            count += 1

    weechat.infolist_free(ilist)

    return weechat.WEECHAT_RC_OK

def get_whitelist(data, option, value):
    """Assign the config values of the whitelist option to the whitelist global
    """

    global whitelist

    t = weechat.config_get_plugin("whitelist")
    whitelist = t.split(",")

    reorder_buffers()

    return weechat.WEECHAT_RC_OK

def set_whitelist(data, option, value):
    """Sets the config directive whitelist
    """
    chanlist = ""

    t = value.replace(' ', '').split(",")
    for chan in t:
        if 2 <= len(chan) and ("#" == chan[0] or "#" == chan[chan.find(".#")+1]):
            chanlist = chanlist + chan + ","

    chanlist=chanlist[:-1] #remove the trailing comma

    weechat.config_set_plugin("whitelist", chanlist)
    weechat.prnt("", "Chanpriority list set to: '{0}'".format(chanlist))

    return weechat.WEECHAT_RC_OK

if weechat.register(SCRIPT_NAME, SCRIPT_AUTHOR, SCRIPT_VERSION, SCRIPT_LICENSE,
        SCRIPT_DESC, "", ""):

    if not weechat.config_is_set_plugin("whitelist"):
        weechat.config_set_plugin("whitelist", "")
    else:
        get_whitelist("", "", "")

def on_join(data, signal, signal_data):
    """Used as callback, called when you join a channel

    If the channel joined is in the whitelist then its buffer will be prioritized
    """

    (chan, network, buffer) = join_meta(data, signal, signal_data)

    if network + "." + chan in whitelist or chan in whitelist:
        weechat.buffer_set(buffer, "number", "2")

    return weechat.WEECHAT_RC_OK

def join_meta(data, signal, signal_data):
    """Get meta info about the JOIN command

    @return a tuple containing the channel and the server joined, the buffer
    that will be opened after joining
    """

    chan = signal_data[signal_data.find("#"):]
    network = signal.partition(",")[0]
    buffer = weechat.buffer_search("irc", network + "." + chan)

    return (chan, network, buffer)

weechat.hook_signal("*,irc_in2_join", "on_join", "data")
weechat.hook_config("plugins.var.python.chanpriority.whitelist",
"get_whitelist", "")

weechat.hook_command(SCRIPT_COMMAND, "Set channel priority", "[#channels]+",
"""When joining channels the buffers will be arranged from high to low priority.
Buffers containing high-priority channels will be in the head of the list:
Positions: 2, 3, 4, ...
They will be followed by low (normal) priority channels.

High-priority channels must be set using this command:
    /chanpriority [#channels]+

The argument must be a list of channels separated by commas, e.g.:
    /chanpriority #chan1, #chan2, #chan3

Channels can also be set using the format: network.#channel:
    /chanpriority #chan1, freenode.#weechat, network.org.#channel

LICENSE: GPL v3

Based on the idea of Flavius Aspra <flavius.as@gmail.com>

Author: Barbu Paul - Gheorghe <paullik.paul@gmail.com>
""", "", "set_whitelist", "")



# -*- coding: utf-8 -*-
#
# Copyright (C) 2013 - 2015  Stefan Wold <ratler@stderr.eu>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# (This script requires WeeChat 0.4.2 or higher).
#
# WeeChat script that enables automatic OTP (OATH-TOTP) support for UnderNET's X
# and Login on Connect (LoC) authentication.
#
# The script will generate an OTP and automatically append it when it
# notices /msg x@channels.undernet.org login <username> <password> command.
# This allows OTP login when using irc.server.*.command to automatically
# sign in to the X service when connecting to an undernet server.
#
#
# Configuration:
#  /set plugins.var.python.undernet-totp.otp_server_names "<server1>,<server2>,..."
#  Set servers for which to automatically enable OTP login
#
# Commands:
#  /uotp [server]
#  Generate an OTP for server, output in core buffer.
#


SCRIPT_NAME    = "undernet_totp"
SCRIPT_AUTHOR  = "Stefan Wold <ratler@stderr.eu>"
SCRIPT_VERSION = "0.3.1"
SCRIPT_LICENSE = "GPL3"
SCRIPT_DESC    = "Enables automatic OTP (OATH-TOTP) support for UnderNET's X and Login on Connect (LoC) authentication."
SCRIPT_COMMAND = "uotp"

HOOKS = {}

SETTINGS = {
    "otp_server_names": ("", "List of undernet server for which to enable OTP, use comma as separator"),
    "debug"           : ("off", "Debug output"),
}

import_ok = True

try:
    import weechat
except ImportError:
    print "This script must be run under WeeChat."
    import_ok = False

try:
    import hmac
    import re
    from base64 import b32decode
    from hashlib import sha1
    from struct import pack, unpack
    from time import time
    from binascii import unhexlify
except ImportError as err:
    print "Missing module(s) for %s: %s" % (SCRIPT_NAME, err)
    import_ok = False


def print_debug(message):
    if weechat.config_get_plugin('debug') == 'on':
        weechat.prnt("", "%s DEBUG: %s" % (SCRIPT_NAME, message))


def unhook(hook):
    global HOOKS

    if hook in HOOKS:
        print_debug('Unhooking %s' % hook)
        weechat.unhook(HOOKS[hook])
        del HOOKS[hook]


def unhook_all(server):
    for hook in [server+'.notice', server+'.modifier', server+'.modifier2']:
        unhook(hook)


def hook_all(server):
    global HOOKS

    notice = server + '.notice'
    modifier = server + '.modifier'
    modifier2 = server + '.modifier2'

    if notice not in HOOKS:
        HOOKS[notice] = weechat.hook_signal("%s,irc_raw_in_notice" % server, "auth_success_cb", server)
    if modifier not in HOOKS:
        HOOKS[modifier] = weechat.hook_modifier("irc_out_privmsg", "totp_login_modifier_cb", server)
    if modifier2 not in HOOKS:
        HOOKS[modifier2] = weechat.hook_modifier("irc_out_pass", "totp_login_modifier_cb", server)


def totp_login_modifier_cb(data, modifier, server, cmd):
    if server == data and server in enabled_servers():
        if re.match(r'(?i)^(PRIVMSG x@channels.undernet.org :login .+ .+|PASS .*)', cmd):
            print_debug("totp_login_modifier_cb(%s)" % cmd)
            otp = generate_totp(server)
            if otp is not None:
                cmd += " %s" % otp
    return cmd


def auth_success_cb(server, signal, signal_data):
    if signal_data.startswith(":X!cservice@undernet.org NOTICE"):
        if re.match(r'^:X!cservice@undernet.org NOTICE .+ :AUTHENTICATION SUCCESSFUL', signal_data):
            unhook_all(server)

    return weechat.WEECHAT_RC_OK


def signal_cb(data, signal, server):
    if server in enabled_servers():
        print_debug('signal_cb(%s)' % signal)
        if signal == 'irc_server_connecting':
            hook_all(server)
        elif signal == 'irc_server_disconnected':
            unhook_all(server)

    return weechat.WEECHAT_RC_OK


def get_otp_cb(data, buffer, server):
    if server:
        server = [server]
    else:
        server = enabled_servers()


    for _server in server:
        otp = generate_totp(_server)
        if otp is not None:
            weechat.prnt("", "%s OTP: %s" % (_server, otp))

    return weechat.WEECHAT_RC_OK


def enabled_servers():
    def server_exists(server):
        print_debug('enabled_servers(%s)' % server)
        if weechat.config_get('irc.server.%s.addresses' % server) is not '':
            return True
        return False

    servers = weechat.config_get_plugin('otp_server_names')
    return [s.strip() for s in servers.split(',') if server_exists(s.strip())]


def generate_totp(server, period=30):
    print_debug('generate_totp(%s)' % server)
    seed = weechat.string_eval_expression("${sec.data.%s_seed}" % server, {}, {}, {})

    if seed is "":
        weechat.prnt("", "No OATH-TOTP secret set, use: /secure set %s_seed <secret>" % server)
        return None

    if len(seed) == 40:  # Assume hex format
        seed = unhexlify(seed)
    else:
        seed = b32decode(seed.replace(" ", ""), True)

    t = pack(">Q", int(time() / period))
    _hmac = hmac.new(seed, t, sha1).digest()
    o = ord(_hmac[19]) & 15
    otp = (unpack(">I", _hmac[o:o+4])[0] & 0x7fffffff) % 1000000

    return '%06d' % otp


if __name__ == "__main__" and import_ok:
    if weechat.register(SCRIPT_NAME, SCRIPT_AUTHOR, SCRIPT_VERSION, SCRIPT_LICENSE,
                        SCRIPT_DESC, "", ""):
        version = weechat.info_get("version_number", "") or 0
        if int(version) < 0x00040200:
            weechat.prnt("", "%s requires WeeChat >= 0.4.2 for secure_data support." % SCRIPT_NAME)
            weechat.command("", "/wait 1ms /python unload %s" % SCRIPT_NAME)

        weechat.hook_command(SCRIPT_COMMAND, "Generate OTP for server", "[server]", "", "%(irc_servers)", "get_otp_cb", "")
        weechat.hook_signal("irc_server_connecting", "signal_cb", "")
        weechat.hook_signal("irc_server_disconnected", "signal_cb", "")

        for option, default_value in SETTINGS.items():
            if weechat.config_get_plugin(option) == "":
                weechat.config_set_plugin(option, default_value[0])
            weechat.config_set_desc_plugin(option, '%s (default: %s)' % (default_value[1], default_value[0]))

        # For now we enable the hooks until it's possible to force script plugins to
        # load before the irc plugin on weechat startup, otherwise the irc_server_connecting signal
        # get missed.
        for _server in enabled_servers():
            hook_all(_server)



# -*- coding: utf-8 -*-
#
# whoissource.py
# Copyright (c) 2014 by Max Teufel <max@teufelsnetz.com>
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
# Display source of "End of WHOIS" numerics
# WeeChat port of http://www.stack.nl/~jilles/irc/whoissource.pl.txt

SCRIPT_NAME    = "whoissource"
SCRIPT_AUTHOR  = "Max Teufel"
SCRIPT_VERSION = "0.0.2"
SCRIPT_LICENSE = "MIT"
SCRIPT_DESC    = "Display source of \"End of WHOIS\" numerics"

try:
    import weechat
    WEECHAT_RC_OK = weechat.WEECHAT_RC_OK
    import_ok = True
except ImportError:
    print "This script must be run under WeeChat."
    import_ok = False

def get_buffer(server, nick):
    msgbuffer = weechat.config_string(weechat.config_get('irc.msgbuffer.%s.%s' % (server, 'whois')))
    if msgbuffer == '':
        msgbuffer = weechat.config_string(weechat.config_get('irc.msgbuffer.%s' % 'whois'))

    if msgbuffer == 'private':
        buffer = weechat.buffer_search('irc', '%s.%s' %(server, nick))
        if buffer != '':
            return buffer
        else:
            msgbuffer = weechat.config_string(weechat.config_get('irc.look.msgbuffer_fallback'))

    if msgbuffer == "current":
        return weechat.current_buffer()
    elif msgbuffer == "weechat":
        return weechat.buffer_search_main()
    else:
        return weechat.buffer_search('irc', 'server.%s' % server)

def endofwhois_cb(data, signal, signal_data):
    whois_source = weechat.info_get("irc_nick_from_host", signal_data)
    server = signal.split(",")[0]
    whois_nick = signal_data.split(" ")[3]
    prefix_network = weechat.prefix('network')
    color_delimiter = weechat.color('chat_delimiters')
    buffer = get_buffer(server, whois_nick)
    if whois_nick == weechat.info_get("irc_nick", server):
        color_nick = weechat.color("chat_nick_self")
    elif weechat.info_get("irc_nick_color", whois_nick) != '':
        color_nick = weechat.info_get("irc_nick_color", whois_nick)
    else:
        color_nick = weechat.color('chat_nick')
    color_reset = weechat.color('reset')
    string = "Got /WHOIS reply from %s" % (whois_source)
    weechat.prnt(buffer, '%s%s[%s%s%s]%s %s' % (prefix_network,
                                                color_delimiter,
                                                color_nick,
                                                whois_nick,
                                                color_delimiter,
                                                color_reset,
                                                string))
    return weechat.WEECHAT_RC_OK

if __name__ == '__main__' and import_ok and weechat.register(SCRIPT_NAME, SCRIPT_AUTHOR, SCRIPT_VERSION, SCRIPT_LICENSE, SCRIPT_DESC, '', ''):
    weechat.hook_signal("*,irc_in2_318", "endofwhois_cb", "")

# vim:set shiftwidth=4 tabstop=4 softtabstop=4 expandtab textwidth=100:



# -*- coding: utf-8 -*-

###
# Copyright (c) 2011, Johannes Nixdorf <mixi@shadowice.org>
# Copyright (c) 2014, Mantas Mikulėnas <grawity@gmail.com>
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice,
#    this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
###

###
# ChangeLog:
#  0.4.3 - grawity:
#   * allow the default player to be set
#  0.4.2 - grawity:
#   * show a better message in "Stopped" state
#  0.4.1 - grawity:
#   * inform the user about MPRIS v1 players
#  0.4 - grawity:
#   * ported to MPRIS v2
#  0.3 - Johannes:
#   * don't excpect every metadata key to be available
#  0.2 - Johannes:
#   * fix an error when the player isn't running
#   * don't display the song if it playing is paused/stopped
#   * fix an error when the player is changed while the plugin is loaded
#   * make fewer dbus calls
#  0.1 - Johannes:
#   * first version
###

from __future__ import print_function

import dbus

try:
    import weechat
    weechat.register('mpris2_np',
                     'Mantas Mikulėnas <grawity@gmail.com>',
                     '0.4.3',
                     'BSD',
                     'Print information on the currently played song',
                     '',
                     '')
except ImportError:
    print('This script must be called from inside weechat')
    raise

BUS_NAME_PREFIX_V2 = 'org.mpris.MediaPlayer2.'
BUS_NAME_PREFIX_V1 = 'org.mpris.'

IF_MPRIS_ROOT   = 'org.mpris.MediaPlayer2'
IF_MPRIS_PLAYER = 'org.mpris.MediaPlayer2.Player'
IF_DBUS_PROP    = 'org.freedesktop.DBus.Properties'

class FubarException(Exception):
    """

    For some reason, reloading either mpris2_np or the python plugin always
    breaks the DBus connection irrecoverably.

    """

def list_players(prefix=BUS_NAME_PREFIX_V2):
    bus = dbus.SessionBus()
    return [name[len(prefix):] for name in bus.list_names()
                               if name.startswith(prefix)]

def get_player(name):
    bus = dbus.SessionBus()
    try:
        return bus.get_object(BUS_NAME_PREFIX_V2 + name, '/org/mpris/MediaPlayer2')
    except TypeError:
        raise FubarException()

def print_info(data, buffer, args):
    err = None
    msg = None

    player_name = args.strip()

    if not player_name:
        player_name = weechat.config_get_plugin('default_player')

    if player_name:
        try:
            player_obj = get_player(player_name)
            prop_if = dbus.Interface(player_obj, IF_DBUS_PROP)

            _props = prop_if.GetAll(IF_MPRIS_ROOT)
            identity = _props.get('Identity', player_name)

            _props = prop_if.GetAll(IF_MPRIS_PLAYER)
            status   = _props.get('PlaybackStatus', 'Stopped')
            metadata = _props.get('Metadata', {})

            if status == 'Stopped':
                msg = u'not listening to anything on %s' % identity
            else:
                artist = u', '.join(metadata.get('xesam:artist', [u'Unknown artist']))
                genre  = u'/'.join(metadata.get('xesam:genre', []))
                album  = metadata.get('xesam:album', None)
                track  = metadata.get('xesam:trackNumber', None)
                title  = metadata.get('xesam:title', u'Unknown title')
                year   = metadata.get('xesam:contentCreated', None)

                msg = u'"%s" by %s' % (title, artist)
                if album:
                    msg += u' from "%s"' % album
                if year:
                    msg += u' (%s)' % year

            msg = msg.encode('utf-8')

        except dbus.exceptions.DBusException as e:
            if e.get_dbus_name() == 'org.freedesktop.DBus.Error.ServiceUnknown':
                err = 'player "%s" is not running' % player_name
            else:
                err = 'DBus error %s: "%s"' % (e.get_dbus_name(), e.get_dbus_message())

        except FubarException:
            err = 'player "%s" is not running (or pydbus is broken again)' % player_name

    else:
        players = list_players()
        if players:
            players.sort()
            err = 'running players: %s' % ', '.join(players)
        else:
            err = 'no MPRIS v2 players are running'
            v1_players = [p for p in list_players(BUS_NAME_PREFIX_V1) if '.' not in p]
            if v1_players:
                err += ' (I found "%s", but it only supports MPRIS v1)' \
                       % ('" and "'.join(v1_players))

    if err:
        weechat.prnt(buffer, 'np: %s' % err)
    else:
        weechat.prnt(buffer, 'np: %s' % msg)
        weechat.command(buffer, 'np: %s' % msg)
    return weechat.WEECHAT_RC_OK

if __name__ == '__main__':
    weechat.hook_command('np',
                         'Print information on the currently played song',
                         '',
                         '',
                         '',
                         'print_info',
                         '')

    if not weechat.config_is_set_plugin('default_player'):
        weechat.config_set_plugin('default_player', '')
        weechat.config_set_desc_plugin('default_player',
            'Player name to use for "/np" (default: "", shows a list)')



# -*- coding: utf-8 -*-
#
# Copyright (C) 2013-2013 KokaKiwi <kokakiwi@kokakiwi.net>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import weechat

SCRIPT_NAME     = "sort_servers"
SCRIPT_AUTHOR   = "KokaKiwi <kokakiwi@kokakiwi.net>"
SCRIPT_VERSION  = "0.1"
SCRIPT_LICENSE  = "GPL3"
SCRIPT_DESC     = "Sort buffers by servers and alphabetically"

if weechat.register(SCRIPT_NAME, SCRIPT_AUTHOR, SCRIPT_VERSION, SCRIPT_LICENSE, SCRIPT_DESC, "", ""):
    weechat.hook_command("sort_servers",
        SCRIPT_DESC,
        "", "", "",
        "sort_servers_cmd",
        ""
    )

def debug(data = ""):
    data = str(data)
    weechat.prnt("", "[sort_servers] %s" % (data))

def command(cmd):
    weechat.command("", cmd)

class SortServers_Buffer:
    def __init__(self, num, server, name):
        self.number = num
        self.server = server
        self.name = name

    @property
    def is_server(self):
        return self.server == 'server'

    @property
    def full_name(self):
        return '%s.%s' % (self.server, self.name)

    @staticmethod
    def sort_key(key):
        return key.name[1:].lower()

    def __str__(self):
        return '%d. %s.%s' % (self.number, self.server, self.name)

    def __repr__(self):
        return str(self)

class SortServers_Server:
    def __init__(self, buf, buffers):
        self.buffer = buf
        self.buffers = buffers

    def sort(self):
        self.buffers = sorted(self.buffers, key = SortServers_Buffer.sort_key)

    @staticmethod
    def sort_key(key):
        return key.buffer.name.lower()

    def __str__(self):
        return '%s(%s)' % (str(self.buffer), ', '.join(map(str, self.buffers)))

    def __repr__(self):
        return str(self)

class SortServersPlugin:
    def get_buffers(self):
        buffers = []

        buffer_pnt = weechat.infolist_get("buffer", "", "")

        while weechat.infolist_next(buffer_pnt):
            buffer_plugin_name = weechat.infolist_string(buffer_pnt, "plugin_name")
            buffer_name = weechat.infolist_string(buffer_pnt, "name")
            buffer_number = weechat.infolist_integer(buffer_pnt, "number")

            buffer_number = int(buffer_number)

            if buffer_plugin_name == 'irc':
                (server, name) = tuple(buffer_name.split('.', 1))

                buf = SortServers_Buffer(buffer_number, server, name)
                buffers.append(buf)

        weechat.infolist_free(buffer_pnt)

        return buffers

    def sort(self):
        buffers = self.get_buffers()

        servers = dict()
        for b in buffers:
            if b.is_server:
                servers[b.name] = SortServers_Server(b, [])
        for b in buffers:
            if not b.is_server:
                server = servers[b.server]
                server.buffers.append(b)

        servers = sorted(servers.values(), key = SortServers_Server.sort_key)
        for server in servers:
            server.sort()

        def move_buffer(src, dst):
            command('/buffer swap %s %s' % (src, dst))

        i = 1
        for server in servers:
            move_buffer(server.buffer.full_name, i)
            i += 1

            for buf in server.buffers:
                move_buffer(buf.full_name, i)
                i += 1

def sort_servers_cmd(data, buf, args):
    ''' Command /sort_servers '''
    current_buffer_name = weechat.buffer_get_string(buf,"name")

    plugin = SortServersPlugin()
    plugin.sort()

    command('/buffer %s' % (current_buffer_name))
    return weechat.WEECHAT_RC_OK



# -*- coding: utf-8 -*-
# Copyright (c) 2009-2010 by xt <xt@bash.no>
# Copyright (c) 2011 by mythmon <mythmon@gmail.com>
# (this script requires WeeChat 0.3.0 or newer)
#
# History:
# 2013-09-16, d33tah
#   version 0.8: Added sort_by_number configuration variable.
# 2013-03-18, mythmon
#   version 0.7: Channels get a bonus based on priority in sorting.
# 2012-02-09, mythmon
#   version 0.6: Allow sorting of channels
# 2010-10-21, xt
#   version 0.5: use ^ for ctrl
# 2010-02-08, bazerka <bazerka@quakenet.org>
#   version 0.4: fix "[Act:]" being shown with no activity, when delimiter
#                is set to " ".
# 2009-05-26, xt <xt@bash.no>
#   version 0.3: only update keydict when key bindings change
# 2009-05-16, xt <xt@bash.no>
#   version 0.2: added support for using keybindings instead of names.
# 2009-05-10, xt <xt@bash.no>
#   version 0.1: initial release.
#
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
'''Hotlist replacement.

Usage: first, put [chanact] in your status bar items.
" weechat.bar.status.items".

Then you can bind keys to buffers with
/key bind meta-w /buffer #weechat

And then it will show as [Act: w] on your status bar.

You can change the order that windows appear in the status bar by setting a
sort rank with
/chanact sort [num]

If you leave off the number, it will print the current sort rank of a buffer.
'''

try:
    import weechat as w
except:
    import sys
    print("This script must be run inside weechat.")
    sys.exit(1)


SCRIPT_NAME    = "chanact"
SCRIPT_AUTHOR  = "xt <xt@bash.no>"
SCRIPT_VERSION = "0.8"
SCRIPT_LICENSE = "GPL3"
SCRIPT_DESC    = "Hotlist replacement, use names and keybindings instead of numbers"

# script options
settings = {
    "lowest_priority"       : '0',
    'message'               : 'Act: ',
    'item_length'           : '8',
    'color_default'         : 'default',
    'color_1'               : 'white',
    'color_2'               : 'cyan',
    'color_3'               : 'lightcyan',
    'color_4'               : 'yellow',
    'color_8'               : 'cyan',
    'use_keybindings'       : 'on',
    'delimiter'             : ',',
    'skip_number_binds'     : 'on',
    'sort_by_number'        : 'off',
}

hooks = (
    ('hotlist_*', 'chanact_update'),
    ('key_bind', 'keydict_update'),
    ('key_unbind', 'keydict_update'),
)

keydict = {}


def chanact_command(data, buffer, args):
    args = args.split(' ')
    if args[0] == 'sort':
        if len(args) == 1:
            s = sort_rank(buffer, 0)
            w.prnt(buffer, 'Sort rank of this buffer: %d' % s)
        elif len(args) == 2:
            s = args[1]
            w.buffer_set(buffer, 'localvar_set_sort', s)
            w.prnt(buffer, 'Sort rank of this buffer set to: %s' % s)

    return w.WEECHAT_RC_OK


def sort_rank(buffer, priority):
    rank = w.buffer_get_string(buffer, 'localvar_sort')
    priority = int(priority) if priority else 0

    if rank:
        return int(rank) + priority
    else:
        return priority


def keydict_update(*args):
    '''Populate a python dictionary with relevant key=>buffer mappings.'''

    global keydict

    keylist = w.infolist_get('key', '', '')
    if w.config_get_plugin('use_keybindings') == 'on':
        while w.infolist_next(keylist):
            key = w.infolist_string(keylist, 'key')
            # we dont want jump sequences
            if 'j' in key:
                continue
            key = key.replace('meta-', '')
            key = key.replace('ctrl-', '^')
            if w.config_get_plugin('skip_number_binds') == 'on':
                # skip entries where buffer number = key, typically entries below 11
                if key.isdigit():
                    continue
            command = w.infolist_string(keylist, 'command')
            # we only care about commands that leads to buffers
            if command.startswith('/buffer'):
                command = command.replace('/buffer ', '')
                buffer = command.lstrip('*')
                keydict[buffer] = key
    w.infolist_free(keylist)
    return w.WEECHAT_RC_OK


def chanact_cb(*args):
    ''' Callback ran on hotlist changes '''
    global keydict

    hotlist = w.infolist_get('hotlist', '', '')

    activity = []
    while w.infolist_next(hotlist):
        priority = w.infolist_integer(hotlist, 'priority')

        if priority < int(w.config_get_plugin('lowest_priority')):
            continue

        int_number = w.infolist_integer(hotlist, 'buffer_number')
        number = str(int_number)
        thebuffer = w.infolist_pointer(hotlist, 'buffer_pointer')
        name = w.buffer_get_string(thebuffer, 'short_name')

        color = w.config_get_plugin('color_default')
        if priority > 0:
            color = w.config_get_plugin('color_%s' %priority)

        if number in keydict:
            number = keydict[number]
            entry = '%s%s%s' % (w.color(color), number, w.color('reset'))
        elif name in keydict:
            name = keydict[name]
            entry = '%s%s%s' % (w.color(color), name, w.color('reset'))
        elif name:
            entry = '%s%s%s:%s%s%s' % (
                    w.color('default'),
                    number,
                    w.color('reset'),
                    w.color(color),
                    name[:int(w.config_get_plugin('item_length'))],
                    w.color('reset'))
        else:
            entry = '%s%s%s' % (
                    w.color(color),
                    number,
                    w.color(reset))

        activity.append((entry, thebuffer, sort_rank(thebuffer, priority), int_number))

    if w.config_get_plugin('sort_by_number') == "on":
        activity.sort(key=lambda t: int(t[3]))
    else:
        activity.sort(key=lambda t: int(t[2]), reverse=True)

    w.infolist_free(hotlist)
    if activity:
        message = w.config_get_plugin('message')
        delim = w.config_get_plugin('delimiter')
        return message + delim.join(a[0] for a in activity)
    else:
        return ''

def chanact_update(*args):
    ''' Hooked to hotlist changes '''

    w.bar_item_update('chanact')

    return w.WEECHAT_RC_OK

if w.register(SCRIPT_NAME, SCRIPT_AUTHOR, SCRIPT_VERSION, SCRIPT_LICENSE,
                    SCRIPT_DESC, '', ''):
    for option, default_value in settings.iteritems():
        if not w.config_is_set_plugin(option):
            w.config_set_plugin(option, default_value)

    for hook, cb in hooks:
        w.hook_signal(hook, cb, '')

    w.bar_item_new('chanact', 'chanact_cb', '')
    keydict_update()


w.hook_command('chanact', 'Manipulate chanact.', 'sort [rank]',
    'sort: Show or set the sort rank of the current buffer.',
    'sort', 'chanact_command', '')



# -*- coding: utf-8 -*-
#
# Copyright (C) 2014  Stefan Wold <ratler@stderr.eu>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# (This script requires WeeChat 0.4.3 or higher).
#
# WeeChat script for responsive layout based on terminal height and width.
#
#
# Source and changes available on GitHUB: https://github.com/Ratler/ratlers-weechat-scripts
#
# Commands:
#  /rlayout


SCRIPT_NAME    = "responsive_layout"
SCRIPT_AUTHOR  = "Stefan Wold <ratler@stderr.eu>"
SCRIPT_VERSION = "0.7"
SCRIPT_LICENSE = "GPL3"
SCRIPT_DESC    = "Responsive layout will automatically apply layouts based on the terminals current size."
SCRIPT_COMMAND = "rlayout"

SETTINGS = {
    "default_nicklist": ("on", "Global setting to always show nicklist when layout switches."),
               "debug": ("off", "Script debug output"),

}

LAYOUT_LIST = []

import_ok = True

try:
    import weechat
except ImportError:
    print "This script must be run under WeeChat."
    import_ok = False

try:
    import re
    from operator import itemgetter
except ImportError as err:
    print "Missing module(s) for %s: %s" % (SCRIPT_NAME, err)
    import_ok = False


def _print(message, buf=""):
    weechat.prnt(buf, "%s: %s" % (SCRIPT_NAME, message))


def _debug(message, buf=""):
    if weechat.config_get_plugin('debug') == 'on':
        weechat.prnt(buf, "+++ %s (debug): %s" % (SCRIPT_NAME, message))


def responsive_cb(data, signal, signal_data):
    term_height = int(weechat.info_get("term_height", ""))
    term_width = int(weechat.info_get("term_width", ""))

    try:
        apply_layout = None
        for layout, width, height in LAYOUT_LIST:
            if term_height <= int(height) or term_width <= int(width):
                apply_layout = layout
                break

        if apply_layout is None:
            # Always apply the last layout if term width/height is larger than configured layouts
            apply_layout = LAYOUT_LIST[-1][0]

        if layout_exist(apply_layout) and not layout_current(apply_layout):
            _print("Applying layout %s" % apply_layout)
            weechat.command("", "/layout apply %s" % apply_layout)
            toggle_nick_list(apply_layout)

        weechat.bar_item_update("rlayout")
    except ValueError:
        _print("Height or width is not in number form, ignoring.")

    return weechat.WEECHAT_RC_OK


def layout_current(layout):
    infolist = weechat.infolist_get("layout", "", "")
    current = False

    while weechat.infolist_next(infolist):
        if weechat.infolist_integer(infolist, "current_layout") == 1 and \
           weechat.infolist_string(infolist, "name") == layout:
            current = True
            break

    weechat.infolist_free(infolist)
    return current


def layout_exist(layout):
    infolist = weechat.infolist_get("layout", "", "")
    found = False

    while weechat.infolist_next(infolist):
        if layout == weechat.infolist_string(infolist, "name"):
            found = True
            break

    weechat.infolist_free(infolist)
    return found


def toggle_nick_list(layout):
    """
    Check configuration whether nick list bar should be on or off for the provided layout.
    """
    value = weechat.config_get_plugin("layout.%s.nicklist" % layout)
    if value == "":
        value = weechat.config_get_plugin("default_nicklist")

    if value == "on":
        weechat.command("", "/bar show nicklist")
    elif value == "off":
        weechat.command("", "/bar hide nicklist")


def rlayouts_list():
    """
    Return a list of configured rlayouts.
    """
    layouts = []
    pattern = re.compile(r"^plugins\.var\.python\.%s\.layout\.(.+)\." % SCRIPT_NAME)
    infolist = weechat.infolist_get("option", "", "plugins.var.python.%s.layout.*" % SCRIPT_NAME)

    while weechat.infolist_next(infolist):
        layout = re.search(pattern, weechat.infolist_string(infolist, "full_name")).groups()
        if layout[0] not in layouts:
            layouts.append(layout[0])

    weechat.infolist_free(infolist)

    return layouts


def update_layout_list():
    """
    Updates global LAYOUT_LIST with a sorted array containing layout tuples, ie (layout_name, width, height)
    """
    global LAYOUT_LIST

    layout_tuples = []

    for layout in rlayouts_list():
        width = weechat.config_get_plugin("layout.%s.width" % layout)
        height = weechat.config_get_plugin("layout.%s.height" % layout)

        if width is not "" and height is not "":
            layout_tuples.append((layout, int(width), int(height)))

    layout_tuples.sort(key=itemgetter(1, 2))
    _debug("update_layout_list(%s)" % layout_tuples)
    LAYOUT_LIST = layout_tuples


def rlayout_cmd_cb(data, buffer, args):
    """
    Callback for /rlayout command.
    """
    if args == "":
        weechat.command("", "/help %s" % SCRIPT_COMMAND)
        return weechat.WEECHAT_RC_OK

    argv = args.strip().split(" ", 1)
    if len(argv) == 0:
        return weechat.WEECHAT_RC_OK

    short_cmds = ["list", "terminal"]
    if argv[0] not in short_cmds and len(argv) < 2:
        _print("Too few arguments for option '%s'." % argv[0])
        return weechat.WEECHAT_RC_OK

    if argv[0] == "size":
        try:
            layout, width, height = argv[1].split(" ")

            if layout_exist(layout):
                weechat.config_set_plugin("layout.%s.width" % layout, width)
                weechat.config_set_plugin("layout.%s.height" % layout, height)
                update_layout_list()
            else:
                _print("Layout '%s' doesn't exist, see /help layout to create one." % layout)
        except ValueError:
            _print("Too few arguments for option '%s'" % argv[0])
    elif argv[0] == "nicklist":
        try:
            layout, nicklist = argv[1].split(" ")

            if layout_exist(layout):
                if nicklist == "on" or nicklist == "off":
                    weechat.config_set_plugin("layout.%s.nicklist" % layout, nicklist)
                else:
                    _print("Invalid argument '%s' for option '%s'." % (nicklist, argv[0]))
            else:
                _print("Layout '%s' doesn't exist, see /help layout to create one." % layout)
        except ValueError:
            _print("Too few arguments for option '%s'" % argv[0])
    elif argv[0] == "default_nicklist" or argv[0] == "debug":
        if argv[1] == "on" or argv[1] == "off":
            weechat.config_set_plugin(argv[0], argv[1])
        else:
            _print("Invalid argument '%s' for option '%s'" % (argv[1], argv[0]))
    elif argv[0] == "remove":
        if argv[1] in rlayouts_list():
            for option in ["width", "height", "nicklist"]:
                weechat.config_unset_plugin("layout.%s.%s" % (argv[1], option))
            _print("Removed rlayout '%s'" % argv[1])
        else:
            _print("Could not remove '%s', rlayout not found." % argv[1])
    elif argv[0] == "list":
        if len(rlayouts_list()) == 0:
            _print("No configuration set.")
        else:
            for rlayout in rlayouts_list():
                width = weechat.config_get_plugin("layout.%s.width" % rlayout)
                height = weechat.config_get_plugin("layout.%s.height" % rlayout)
                nicklist = weechat.config_get_plugin("layout.%s.nicklist" % rlayout)
                msg = "[%s] width: %s, height: %s" % (rlayout, width, height)
                if nicklist is not "":
                    msg += ", nicklist: %s" % nicklist
                _print(msg)
    elif argv[0] == "terminal":
        term_height = int(weechat.info_get("term_height", ""))
        term_width = int(weechat.info_get("term_width", ""))
        _print("Current terminal width x height is: %s x %s" % (term_width, term_height))

    return weechat.WEECHAT_RC_OK


def rlayout_bar_cb(data, item, window):
    infolist = weechat.infolist_get("layout", "", "")
    layout = None

    while weechat.infolist_next(infolist):
        if weechat.infolist_integer(infolist, "current_layout") == 1:
            layout = weechat.infolist_string(infolist, "name")
            break

    weechat.infolist_free(infolist)

    if layout is None:
        return ""
    else:
        term_height = int(weechat.info_get("term_height", ""))
        term_width = int(weechat.info_get("term_width", ""))

    return "%s (%sx%s)" % (layout, term_width, term_height)


def rlayout_completion_bool_cb(data, completion_item, buffer, completion):
    for bool in ("on", "off"):
        weechat.hook_completion_list_add(completion, bool, 0, weechat.WEECHAT_LIST_POS_SORT)
    return weechat.WEECHAT_RC_OK


def rlayout_completion_layout_list_cb(data, completion_item, buffer, completion):
    for rlayout in rlayouts_list():
        weechat.hook_completion_list_add(completion, rlayout, 0, weechat.WEECHAT_LIST_POS_SORT)
    return weechat.WEECHAT_RC_OK


def config_cb(data, option, value):
    update_layout_list()
    return weechat.WEECHAT_RC_OK


if __name__ == "__main__" and import_ok:
    if weechat.register(SCRIPT_NAME, SCRIPT_AUTHOR, SCRIPT_VERSION, SCRIPT_LICENSE, SCRIPT_DESC, "", ""):
        version = weechat.info_get("version_number", "") or 0
        if int(version) < 0x00040300:
            _print("Requires WeeChat >= 0.4.3 for terminal height and width support.")
            weechat.command("", "/wait 1ms /python unload %s" % SCRIPT_NAME)

        weechat.hook_command(SCRIPT_COMMAND,
                             "WeeChat responsive layout configuration",
                             "size <layout> <width> <height> || nicklist <layout> <on|off> || default_nicklist <on|off>"
                             " || remove <layout> || list || terminal || debug <on|off>",
                             "            size: set max size (width and height) for layout to be automatically applied\n"
                             "        nicklist: show or hide nicklist bar when layout is automatically applied\n"
                             "default_nicklist: default show or hide nicklist bar if not configured per layout\n"
                             "          remove: remove settings for responsive layout\n"
                             "            list: list current configuration\n"
                             "        terminal: list current terminal width and height\n"
                             "           debug: print script debug output\n\n"
                             "To get current layout and terminal dimensions in your bar, use 'rlayout' bar item.",
                             "size %(layouts_names)"
                             " || nicklist %(layouts_names) %(rlayout_bool_value)"
                             " || default_nicklist %(rlayout_bool_value)"
                             " || remove %(rlayouts_names)"
                             " || list"
                             " || terminal"
                             " || debug %(rlayout_bool_value)",
                             "rlayout_cmd_cb",
                             "")

        # Default settings
        for option, default_value in SETTINGS.items():
            if weechat.config_get_plugin(option) == "":
                weechat.config_set_plugin(option, default_value[0])
            weechat.config_set_desc_plugin(option, '%s (default: %s)' % (default_value[1], default_value[0]))

        weechat.hook_completion("rlayout_bool_value", "list of bool values", "rlayout_completion_bool_cb", "")
        weechat.hook_completion("rlayouts_names", "list of rlayouts", "rlayout_completion_layout_list_cb", "")
        weechat.hook_config("plugins.var.python.responsive_layout.layout.*", "config_cb", "")
        weechat.bar_item_new("rlayout", "rlayout_bar_cb", "")
        update_layout_list()
        hook = weechat.hook_signal("signal_sigwinch", "responsive_cb", "")



# -*- coding: utf-8 -*-
#
# Copyright (C) 2010-2012 Sebastien Helleu <flashcode@flashtux.org>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

#
# Antispam for DCC file/chat requests.
# (this script requires WeeChat 0.3.0 or newer)
#
# History:
#
# 2012-01-03, Sebastien Helleu <flashcode@flashtux.org>:
#     version 0.2: make script compatible with Python 3.x
# 2010-02-09, Sebastien Helleu <flashcode@flashtux.org>:
#     version 0.1: initial release
#

import_ok = True

try:
    import weechat
except:
    print("This script must be run under WeeChat.")
    print("Get WeeChat now at: http://www.weechat.org/")
    import_ok = False

import re

SCRIPT_NAME    = "dcc_antispam"
SCRIPT_AUTHOR  = "Sebastien Helleu <flashcode@flashtux.org>"
SCRIPT_VERSION = "0.2"
SCRIPT_LICENSE = "GPL3"
SCRIPT_DESC    = "Antispam for DCC file/chat requests"

# script options
dcc_settings = {
    "display" : "on",          # display blocked dcc on core buffer?
    # comma separated lists:
    "server"  : "",            # restricts anti-spam on these servers (internal names)
    "name"    : "",            # list of names (regex) to block
    "ip"      : "^0.0.0.0",    # list of IPs (regex) to block
    "port"    : "^0$,^\D+$",   # list of ports (regex) to block
    "size"    : "0",           # list of sizes to block (should be "0" or empty string)
}
dcc_settings_regex_allowed = [ "name", "ip", "port" ]
dcc_settings_regex = {}

# DCC message looks like:
#   SEND:
#     :nick!user@host.com PRIVMSG mynick :\01DCC SEND filename.txt 2130706433 40017 123456\01
#                                                     \----------/ \--------/ \---/ \----/
#                                                         name         ip     port   size
#   CHAT:
#     :nick!user@host.com PRIVMSG mynick :\01DCC CHAT chat 2130706433 40017\01
#                                                          \--------/ \---/
#                                                              ip     port

dcc_regex_dcc_send = re.compile(r"\S+ PRIVMSG \S+ :\01DCC (SEND|CHAT) ([^\01]+)\01")


def dcc_is_blocked(dcc):
    """ Return True if dcc is blocked, False if dcc is not blocked. """
    global dcc_settings_regex
    for key, value in dcc.items():
        if key in dcc_settings_regex:
            if weechat.config_get_plugin(key) != "":
                for regex in dcc_settings_regex[key]:
                    if regex.search(value):
                        return True
        else:
            for str in weechat.config_get_plugin(key).split(","):
                if value == str:
                    return True
    # dcc NOT blocked
    return False

def dcc_parse(type, message):
    """ Parse dcc request, return dict with values. """
    dcc = {}
    for arg in [ "size", "port", "ip" ]:
        if type == "CHAT" and arg == "size":
            continue;
        pos = message.rfind(" ")
        if pos < 0:
            # problem with parsing, dcc is not blocked
            return None
        dcc[arg] = message[pos+1:]
        message = message[:pos]
    if type == "SEND":
        if message.startswith("\"") and message.endswith("\""):
            message = message[1:len(message)-1]
        dcc["name"] = message
    ip = int(dcc["ip"])
    dcc["ip"] = "%d.%d.%d.%d" % (ip >> 24, (ip >> 16) & 0xff, (ip >> 8) & 0xff, ip & 0xff)
    return dcc

def dcc_privmsg_modifier_cb(data, modifier, modifier_data, string):
    """ Modifier callback for IRC PRIVMSG messages. """
    match = dcc_regex_dcc_send.match(string)
    if match:
        server = weechat.config_get_plugin("server")
        if server == "" or modifier_data in server.split(","):
            dcctype = match.group(1)
            dcc = dcc_parse(dcctype, match.group(2))
            if dcc and dcc_is_blocked(dcc):
                if weechat.config_get_plugin("display") == "on":
                    if dcctype == "CHAT":
                        weechat.prnt("", "%s: dcc blocked (chat from %s, ip: %s, port: %s)" %
                                     (SCRIPT_NAME, weechat.info_get("irc_nick_from_host", string),
                                      dcc["ip"], dcc["port"]))
                    else:
                        weechat.prnt("", "%s: dcc blocked (file: '%s' from %s, size: %s bytes, ip: %s, port: %s)" %
                                     (SCRIPT_NAME, dcc["name"], weechat.info_get("irc_nick_from_host", string),
                                      dcc["size"], dcc["ip"], dcc["port"]))
                return ""

    return string

def dcc_build_regex():
    """ Build regex for all options. """
    global dcc_settings_regex_allowed, dcc_settings_regex
    for key in dcc_settings_regex_allowed:
        dcc_settings_regex[key] = []
        try:
            for string in weechat.config_get_plugin(key).split(","):
                dcc_settings_regex[key].append(re.compile(string, re.I))
        except:
            weechat.prnt("", "%s%s: error compiling regex in '%s' (option %s)" %
                         (weechat.prefix("error"), SCRIPT_NAME,
                          weechat.config_get_plugin(key), key))
            dcc_settings_regex[key] = []

def dcc_config_cb(data, option, value):
    """ Callback called when a script option is changed. """
    dcc_build_regex()
    return weechat.WEECHAT_RC_OK

if __name__ == "__main__" and import_ok:
    if weechat.register(SCRIPT_NAME, SCRIPT_AUTHOR, SCRIPT_VERSION,
                        SCRIPT_LICENSE, SCRIPT_DESC,
                        "", ""):
        # set default settings
        for option, default_value in dcc_settings.items():
            if not weechat.config_is_set_plugin(option):
                weechat.config_set_plugin(option, default_value)
        dcc_build_regex()
        # modifier for privmsg messages
        weechat.hook_modifier("irc_in_privmsg", "dcc_privmsg_modifier_cb", "")
        # config
        weechat.hook_config("plugins.var.python." + SCRIPT_NAME + ".*", "dcc_config_cb", "")



#!/usr/bin/python
# -*- encoding: utf-8 -*-
#
# Copyright (c) 2014-2016 by deflax <daniel@deflax.net>
# Copyright (c) 2015 Christopher Stewart
# Copyright (c) 2016 by esch
#
#       Weechat WeatherBot using WUnderground API
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

SCRIPT_NAME    = "weatherbot"
SCRIPT_AUTHOR  = "deflax <daniel@deflax.net>"
SCRIPT_VERSION = "0.9"
SCRIPT_LICENSE = "GPL3"
SCRIPT_DESC    = "WeatherBot using the WeatherUnderground API"

try:
    import weechat as w, json, datetime
    import_ok = True
except ImportError:
    print "This script must be run under WeeChat."
    import_ok = False

helptext = """
Get your own API key from http://www.wunderground.com/weather/api/
and act as weatherbot :)

\o/ hugs to all friends from #clanchill@quakenet and initlab.org \o/
"""
default_options = {"enabled": "on",
                   "units": "metric",
                   "weather_trigger": "!weather",
                   "forecast_trigger": "!forecast",
                   "apikey": "0000000000000"}

plugin_config = "plugins.var.python.{}".format(SCRIPT_NAME)

def weebuffer(reaction):
    rtnbuf = "{},{}".format(kserver, kchannel)
    buffer = w.info_get("irc_buffer", rtnbuf)
    botnick = w.info_get("irc_nick", kserver)
    if kchannel == botnick:
        #priate
        command = "msg {} {}".format(knick, reaction)
    else:
        #channel
        command = "msg {} {}".format(kchannel, reaction)

    #w.prnt("", command)
    cmdprefix = "/"
    w.command(buffer, cmdprefix + command)


def wu_autoc(data, command, return_code, out, err):
    """ weather underground auto search """
    global jname
    if return_code == w.WEECHAT_HOOK_PROCESS_ERROR:
        w.prnt("", "Error with command `%s'" % command)
        return w.WEECHAT_RC_OK
    if return_code > 0:
        w.prnt("", "return_code = %d" % return_code)
    if err != "":
        w.prnt("", "stderr: %s" % err)
    if out != "":
        i = json.loads(out)
        try:
            loc = next((l for l in i["RESULTS"] if l["type"] == "city"), None)
            if loc is None:
                weebuffer("Unable to locate query.")
                return w.WEECHAT_RC_OK
        except:
            weebuffer("Invalid query. Try again.")
            return w.WEECHAT_RC_OK

        jname = loc["name"]
        location = loc["l"]
        prefix = "[weatherbot] mode:"
        if mode == "conditions":
            cond_url = "url:http://api.wunderground.com/api/{}/conditions{}.json".format(options["apikey"], location)
            w.prnt("", '{} {} {}'.format(prefix, mode, location))
            w.hook_process(cond_url, 30 * 1000, "wu_cond", "")

        if mode == "forecast":
            fore_url = "url:http://api.wunderground.com/api/{}/forecast{}.json".format(options["apikey"], location)
            w.prnt("", '{} {} {}'.format(prefix, mode, location))
            w.hook_process(fore_url, 30 * 1000, "wu_fore", "")

    return w.WEECHAT_RC_OK


def wu_fore(data, command, return_code, out, err):
    """ wu forecast """
    if return_code == w.WEECHAT_HOOK_PROCESS_ERROR:
        w.prnt("", "Error with command '%s'" % command)
        return w.WEECHAT_RC_OK
    if return_code > 0:
        w.prnt("", "return_code = %d" % return_code)
    if err != "":
        w.prnt("", "stderr: %s" % err)
    if out != "":
        j = json.loads(out)
        try:
            error_type = j["response"]["error"]["type"]
            if error_type == "invalidquery":
                weebuffer("Error. Try again.")
                return w.WEECHAT_RC_OK
            elif error_type == "keynotfound":
                weebuffer("Invalid API key.")
                return w.WEECHAT_RC_OK
        except KeyError:
            pass

	if options["units"] == "metric":
            fcttext = 'fcttext_metric'
        else:
            fcttext = 'fcttext'

	#hour_string = j['forecast']['txt_forecast']['date']
	#hour_stripped = hour_string.rpartition(" ")[0]
        now = datetime.datetime.now()
        strhour = now.strftime('%H')
        hour = int(strhour)

        reaction = '[{}] [*'.format(jname)

        #if earlier than 1600 show forecast for today and tonight
        if hour < 16:
            reaction += j['forecast']['txt_forecast']['forecastday'][0]['title'] + '*]: '
            reaction += j['forecast']['txt_forecast']['forecastday'][0][fcttext] + ' [*'
            reaction += j['forecast']['txt_forecast']['forecastday'][1]['title'] + '*]: '
            reaction += j['forecast']['txt_forecast']['forecastday'][1][fcttext]
        #between 1600 and 2100 show forecast for tonight and tomorrow
        elif 16 <= hour <= 21:
            reaction += j['forecast']['txt_forecast']['forecastday'][1]['title'] + '*]: '
            reaction += j['forecast']['txt_forecast']['forecastday'][1][fcttext] + ' [*'
            reaction += j['forecast']['txt_forecast']['forecastday'][2]['title'] + '*]: '
            reaction += j['forecast']['txt_forecast']['forecastday'][2][fcttext]

        #after 2100 show forecast for tomorrow and tomorrow night
        elif hour > 21:
            reaction += j['forecast']['txt_forecast']['forecastday'][2]['title'] + '*]: '
            reaction += j['forecast']['txt_forecast']['forecastday'][2][fcttext] + ' [*'
            reaction += j['forecast']['txt_forecast']['forecastday'][3]['title'] + '*]: '
            reaction += j['forecast']['txt_forecast']['forecastday'][3][fcttext]
        weebuffer(reaction)

    return w.WEECHAT_RC_OK


def wu_cond(data, command, return_code, out, err):
    """ wu condition """
    if return_code == w.WEECHAT_HOOK_PROCESS_ERROR:
        w.prnt("", "Error with command '%s'" % command)
        return w.WEECHAT_RC_OK
    if return_code > 0:
        w.prnt("", "return_code = %d" % return_code)
    if err != "":
        w.prnt("", "stderr: %s" % err)
    if out != "":
        j = json.loads(out)
        try:
            error_type = j["response"]["error"]["type"]
            if error_type == "invalidquery":
                weebuffer("Error. Try again.")
                return w.WEECHAT_RC_OK
            elif error_type == "keynotfound":
                weebuffer("Invalid API key.")
                return w.WEECHAT_RC_OK
        except KeyError:
            pass

        co = j["current_observation"]
        reaction = "[{}] {}. Temp is ".format(jname, co["weather"])

        if options["units"] == "metric":
            temp_unit = "C"
            wind_unit = "kph"
        else:
            temp_unit = "F"
            wind_unit = "mph"

        temp = co["temp_{}".format(temp_unit.lower())]
        like = co["feelslike_{}".format(temp_unit.lower())]
        if abs(int(float(temp)) - int(float(like))) > 2:
            reaction += "{0}°{1} but feels like {2}°{1}.".format(temp, temp_unit, like)
        else:
            reaction += "{}°{}.".format(temp, temp_unit)

        wind_speed = co["wind_{}".format(wind_unit)]
        if wind_speed > 0:
            reaction += " {} wind: {} {}.".format(co["wind_dir"], wind_speed, wind_unit)

        humid = co["relative_humidity"]
        if int(humid[:-1]) > 50:
            reaction += " Humidity: {}.".format(co["relative_humidity"])

        weebuffer(reaction)
    return w.WEECHAT_RC_OK


def triggerwatch(data, buffer, args):
    global kserver, kchannel, knick, mode
    if options["enabled"] == "on":
        try:
            null, srvmsg = args.split(" PRIVMSG ", 1)
        except:
            return w.WEECHAT_RC_OK

        try:
            kchannel, query = srvmsg.split(" :{} ".format(options["weather_trigger"]), 1)
            mode = "conditions"
        except ValueError:
            try:
                kchannel, query = srvmsg.split(" :{} ".format(options["forecast_trigger"]), 1)
                mode = "forecast"
            except ValueError:
                return w.WEECHAT_RC_OK

        kserver = str(buffer.split(",", 1)[0])
        knick = w.info_get("irc_nick_from_host", args)
        query = query.replace(" ", "%20")

        autoc_url = "url:http://autocomplete.wunderground.com/aq?query={}&format=JSON".format(query)
        w.hook_process(autoc_url, 30 * 1000, "wu_autoc", "")

    return w.WEECHAT_RC_OK


def config_cb(data, option, value):
    """Callback called when a script option is changed."""
    opt = option.split(".")[-1]
    options[opt] = value
    return w.WEECHAT_RC_OK


def get_option(option):
    """Returns value of w.option."""
    return w.config_string(w.config_get("{}.{}".format(plugin_config, option)))


if __name__ == "__main__" and import_ok:
    if w.register(SCRIPT_NAME, SCRIPT_AUTHOR, SCRIPT_VERSION, SCRIPT_LICENSE, SCRIPT_DESC, "", ""):
        #set options
        for option,value in default_options.items():
            if not w.config_is_set_plugin(option):
                w.config_set_plugin(option, value)

        options = {"enabled": get_option("enabled"),
            "units": get_option("units"),
            "weather_trigger": get_option("weather_trigger"),
            "forecast_trigger": get_option("forecast_trigger"),
            "apikey": get_option("apikey")
        }

        if options["apikey"] == "0000000000000":
            w.prnt("", "Your API key is not set. Please sign up at www.wunderground.com/weather/api and set plugins.var.python.weatherbot.*")

        #start
        w.hook_signal("*,irc_in_privmsg", "triggerwatch", "data")
        w.hook_config("{}.*".format(plugin_config), "config_cb", "")
